@@ -169,7 +169,7 @@ XwFormClassRec XwformClassRec =
 {
    {
       (WidgetClass) &XwmanagerClassRec, /* superclass	         */	
-      "Form",                           /* class_name	         */	
+      "XwForm",                         /* class_name	         */	
       sizeof(XwFormRec),                /* widget_size	         */	
       NULL,                             /* class_initialize      */    
       NULL,                             /* class_part_initialize */    
