@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* colordefs.h 								   */
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University       	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -9,7 +9,7 @@
 /* And ApplicationDataPtr in xcircuit.h					   */
 /*-------------------------------------------------------------------------*/
 
-#define NUMBER_OF_COLORS	13
+#define NUMBER_OF_COLORS	16
 
 #define BACKGROUND	appcolors[0]
 #define FOREGROUND	appcolors[1]
@@ -22,8 +22,14 @@
 #define OFFBUTTONCOLOR	appcolors[8]
 #define AUXCOLOR	appcolors[9]
 #define BARCOLOR	appcolors[10]
-#define BBOXCOLOR	appcolors[11]
-#define PARAMCOLOR	appcolors[12]
+#define PARAMCOLOR	appcolors[11]
+
+/* The rest of the colors are layout colors, not GUI colors */
+
+#define BBOXCOLOR	appcolors[12]
+#define LOCALPINCOLOR	appcolors[13]
+#define GLOBALPINCOLOR	appcolors[14]
+#define INFOLABELCOLOR	appcolors[15]
 
 #define DEFAULTCOLOR	-1		/* Inherits color of parent */
 #define DOFORALL	-2		/* All elements inherit same color */
