@@ -1,16 +1,11 @@
 /*-------------------------------------------------------------------------*/
 /* filelist.c --- Xcircuit routines for the filelist widget		   */ 
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <sys/types.h>
 
 #ifdef HAVE_DIRENT_H
@@ -31,10 +26,14 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
 #include "Xw/WorkSpace.h"
 #include "Xw/TextEdit.h"
 #include "Xw/Toggle.h"
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -58,8 +57,11 @@
 /* Global Variable definitions						   */
 /*-------------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER
+extern Tcl_Interp *xcinterp;
+#endif
+
 extern Display    *dpy;
-extern Window     win;
 extern Clientdata areastruct;
 extern ApplicationData appdata;
 extern int 	  *appcolors;
@@ -90,11 +92,11 @@ int fcompare(const void *a, const void *b)
 /* Routines for drawing a box around the currently selected file	   */
 /*-------------------------------------------------------------------------*/
 
-void dragfilebox(Widget w, caddr_t clientdata, XMotionEvent *event)
+void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
 {
    short filenum;
    int twidth;
-   Window lwin = XtWindow(w);
+   Window lwin = xcWindow(w);
 
    filenum = (event->y - 10 + FILECHARHEIGHT) / FILECHARHEIGHT + flstart - 1;
    if (filenum < 0) filenum = 0;
@@ -121,9 +123,14 @@ void dragfilebox(Widget w, caddr_t clientdata, XMotionEvent *event)
 /* Begin tracking the cursor position relative to the files in the list    */
 /*-------------------------------------------------------------------------*/
 
-void startfiletrack(Widget w, caddr_t clientdata, XCrossingEvent *event)
+void startfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 {
-   XtAddEventHandler(w, PointerMotionMask, False, (XtEventHandler)dragfilebox, NULL);
+#ifdef TCL_WRAPPER
+   Tk_CreateEventHandler(w, PointerMotionMask,
+		(Tk_EventProc *)xctk_dragfilebox, (ClientData)w);
+#else
+   xcAddEventHandler(w, PointerMotionMask, False, (xcEventHandler)dragfilebox, NULL);
+#endif
 
    XSetFunction(dpy, areastruct.gc, GXcopy);
    XSetForeground(dpy, areastruct.gc, AUXCOLOR);
@@ -142,16 +149,21 @@ void startfiletrack(Widget w, caddr_t clientdata, XCrossingEvent *event)
 /* Stop tracking the cursor and return to default state			   */
 /*-------------------------------------------------------------------------*/
 
-void endfiletrack(Widget w, caddr_t clientdata, XCrossingEvent *event)
+void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 {
-   Window lwin = XtWindow(w);
+   Window lwin = xcWindow(w);
 
    XDrawRectangle(dpy, lwin, areastruct.gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
-   XtRemoveEventHandler(w, Button1MotionMask | Button2MotionMask,
-	False, (XtEventHandler)dragfilebox, NULL);
+#ifdef TCL_WRAPPER
+   Tk_DeleteEventHandler(w, PointerMotionMask,
+		(Tk_EventProc *)xctk_dragfilebox, (ClientData)w);
+#else
+   xcRemoveEventHandler(w, PointerMotionMask, False,
+		(xcEventHandler)dragfilebox, NULL);
+#endif
 
    /* Restore graphics state values */
    XSetForeground(dpy, areastruct.gc, areastruct.gccolor);
@@ -224,9 +236,8 @@ void findcrashfiles()
 
    while ((dp = readdir(cwd)) != NULL) {
       sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
-      if (!strncmp(_STR + 5, "XC", 2)) {
-         stat(_STR, &sbuf);
-         if (sbuf.st_uid == userid) {
+      if (!strncmp(_STR + strlen(xobjs.tempdir) + 1, "XC", 2)) {
+         if ((!stat(_STR, &sbuf)) && (sbuf.st_uid == userid)) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
 	       recent = sbuf.st_ctime;
 	       strcpy(_STR2, _STR);
@@ -243,7 +254,17 @@ void findcrashfiles()
       /* Warn user of existing tempfile, and ask user if file	*/
       /* should be recovered immediately.			*/
 
-      getfile(NULL, Number(RECOVER), NULL);   /* Crash recovery mode */
+#ifdef TCL_WRAPPER
+      char *cfile = getcrashfilename();
+      sprintf(_STR, ".query.title.field configure -text "
+		"\"Recover file \'%s\'?\"", 
+		(cfile == NULL) ? "(unknown)" : cfile);
+      Tcl_Eval(xcinterp, _STR);
+      Tcl_Eval(xcinterp, ".query.bbar.okay configure -command "
+		"{filerecover; wm withdraw .query}; wm deiconify .query");
+#else
+      getfile(NULL, (pointertype)RECOVER, NULL);   /* Crash recovery mode */
+#endif
    }
 }  
 
@@ -266,13 +287,13 @@ Boolean match_filter(char *fname, char *filter)
 /* Make a list of the files in the list widget window			*/
 /*----------------------------------------------------------------------*/
 
-void listfiles(Widget w, popupstruct *okaystruct, caddr_t calldata)
+void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
    XGCValues	values;
    Arg wargs[2];
    DIR *cwd;
    char *filter;
-   Window lwin = XtWindow(w);
+   Window lwin = xcWindow(w);
    short allocd = INITDIRS;
    short n = 0;
    struct direct *dp;
@@ -286,12 +307,19 @@ void listfiles(Widget w, popupstruct *okaystruct, caddr_t calldata)
       values.foreground = FOREGROUND;
       values.font = appdata.filefont->fid;
       values.function = GXcopy;
-      sgc = XCreateGC(dpy, lwin, GCForeground | GCFont | GCFunction, &values);
+      values.graphics_exposures = False;
+      sgc = XCreateGC(dpy, lwin, GCForeground | GCFont | GCFunction
+		| GCGraphicsExposures, &values);
    }
 
+#ifdef TCL_WRAPPER
+   textwidth = Tk_Width(w);
+   textheight = Tk_Height(w);
+#else
    XtnSetArg(XtNwidth, &textwidth);
    XtnSetArg(XtNheight, &textheight);
    XtGetValues(w, wargs, n);
+#endif
 
    /* Generate a new flistpix pixmap if currently nonexistent */
 
@@ -334,7 +362,7 @@ void listfiles(Widget w, popupstruct *okaystruct, caddr_t calldata)
 	    /* record the type of file */
 	
 	    sprintf(_STR2, "%s%s", cwdname, dp->d_name); 
-	    stat(_STR2, &statbuf);
+	    if (stat(_STR2, &statbuf)) continue;
 	    if ((statbuf.st_mode & S_IFDIR) != 0) /* is a directory */
 	       files[flfiles].filetype = DIRECTORY;
 	    else if (match_filter(dp->d_name, filter))
@@ -366,10 +394,11 @@ void listfiles(Widget w, popupstruct *okaystruct, caddr_t calldata)
 
       qsort((void *)files, (size_t)flfiles, sizeof(fileliststruct), fcompare);
 
-      pixheight = flfiles * FILECHARHEIGHT + 20;
+      pixheight = flfiles * FILECHARHEIGHT + 25;
+      if (pixheight < textheight) pixheight = textheight;
 
-      flistpix = XCreatePixmap(dpy, win, textwidth, pixheight,
-	   DefaultDepthOfScreen(XtScreen(w)));
+      flistpix = XCreatePixmap(dpy, areastruct.areawin, textwidth, pixheight,
+	   DefaultDepthOfScreen(xcScreen(w)));
 
       /* Write the filenames onto the pixmap */
 
@@ -406,10 +435,26 @@ void listfiles(Widget w, popupstruct *okaystruct, caddr_t calldata)
 /* size accordingly.							   */
 /*-------------------------------------------------------------------------*/
 
-void newfilelist(Widget w, popupstruct *okaystruct)
+void newfilelist(xcWidget w, popupstruct *okaystruct)
 {
    short n;
-   Widget textw = okaystruct->textw;
+   xcWidget textw = okaystruct->textw;
+
+#ifdef TCL_WRAPPER
+   int bval;
+   int result;
+   char *cstr = Tcl_GetVar(xcinterp, "filter", TCL_NAMESPACE_ONLY);
+   if (cstr == NULL) {
+      Wprintf("Error: No variable $filter in Tcl!");
+      return;
+   }
+   result = Tcl_GetBoolean(xcinterp, cstr, &bval);
+   if (result != TCL_OK) {
+      Wprintf("Error: Bad variable $filter in Tcl!");
+      return;  
+   }
+   areastruct.filefilter = bval;
+#endif
 
    for (n = 0; n < flfiles; n++)
       free(files[n].filename);
@@ -419,30 +464,42 @@ void newfilelist(Widget w, popupstruct *okaystruct)
    flistpix = (Pixmap)NULL;
    flstart = 0;
    listfiles(w, okaystruct, NULL);
-   showlscroll(XtNameToWidget(XtParent(w), "LScroll"), NULL, NULL);
+#ifdef TCL_WRAPPER
+   showlscroll(Tk_NameToWindow(xcinterp, ".filelist.listwin.sb", w), NULL, NULL);
+   Tcl_Eval(xcinterp, ".filelist.textent.txt delete 0 end");
+   sprintf(_STR2, ".filelist.textent.txt insert 0 %s", cwdname);
+   Tcl_Eval(xcinterp, _STR2);
+#else
+   showlscroll(XtNameToWidget(xcParent(w), "LScroll"), NULL, NULL);
    XwTextClearBuffer(textw);
    XwTextInsert(textw, cwdname);
    XwTextResize(textw);
+#endif
 }
 
 /*-------------------------------------------------------------------------*/
 /* Button press handler for file list window				   */
 /*-------------------------------------------------------------------------*/
 
-void fileselect(Widget w, popupstruct *okaystruct, XButtonEvent *event)
+void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
    Arg wargs[2];
-   Window lwin = XtWindow(w);
+   Window lwin = xcWindow(w);
    Dimension textwidth, textheight;
    short filenum, n = 0;
-   char *tbuf;
-   Widget textw = okaystruct->textw;
+   char *tbuf, *ebuf;
+   xcWidget textw = okaystruct->textw;
 
    flcurrent = -1;
 
+#ifdef TCL_WRAPPER
+   textwidth = Tk_Width(w);
+   textheight = Tk_Height(w);
+#else
    XtnSetArg(XtNwidth, &textwidth);
    XtnSetArg(XtNheight, &textheight);
    XtGetValues(w, wargs, n);
+#endif
 
    /* third mouse button cancels selection and reverts buffer to cwd name */
 
@@ -476,9 +533,17 @@ void fileselect(Widget w, popupstruct *okaystruct, XButtonEvent *event)
 
       /* . . .and append it to the text field */
 
+#ifdef TCL_WRAPPER
+      Tcl_Eval(xcinterp, ".filelist.textent.txt get");
+      ebuf = (char *)Tcl_GetStringResult(xcinterp);
+      tbuf = (char *)malloc((strlen(ebuf) +
+		strlen(files[filenum].filename) + 6) * sizeof(char));
+#else
+      ebuf = (char *)XwTextCopyBuffer(textw);
       tbuf = (char *)malloc((XwTextGetLastPos(textw)
 	     + strlen(files[filenum].filename) + 5) * sizeof(char));
-      strcpy(tbuf, (char *)XwTextCopyBuffer(textw)); 
+#endif
+      strcpy(tbuf, ebuf);
 
       /* add a comma if there is already text in the destination buffer */
 
@@ -495,9 +560,15 @@ void fileselect(Widget w, popupstruct *okaystruct, XButtonEvent *event)
  	 }
       }
       strcat(tbuf, files[filenum].filename);
+#ifdef TCL_WRAPPER
+      Tcl_Eval(xcinterp, ".filelist.textent.txt delete 0 end");
+      sprintf(_STR2, ".filelist.textent.txt insert 0 %s", tbuf);
+      Tcl_Eval(xcinterp, _STR2);
+#else
       XwTextClearBuffer(textw);
       XwTextInsert(textw, tbuf);
       XwTextResize(textw);
+#endif
       free(tbuf);
    }
    else {  /* move to new directory */
@@ -530,24 +601,28 @@ void fileselect(Widget w, popupstruct *okaystruct, XButtonEvent *event)
 /* Scrollbar handler for file list widget				   */
 /*-------------------------------------------------------------------------*/
 
-void showlscroll(Widget w, caddr_t clientdata, caddr_t calldata)
+void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    Arg wargs[2];
-   Window swin = XtWindow(w);
+   Window swin = xcWindow(w);
    Dimension swidth, sheight;
    int pstart, pheight, finscr;
    short n = 0;
 
-
+#ifdef TCL_WRAPPER
+   swidth = Tk_Width(w);
+   sheight = Tk_Height(w);
+#else
    XtnSetArg(XtNwidth, &swidth);
    XtnSetArg(XtNheight, &sheight);
    XtGetValues(w, wargs, n);
+#endif
 
    XClearWindow(dpy, swin);
 
    if (flfiles > 0) {	/* no files, no bar */
 
-      finscr = LISTHEIGHT / FILECHARHEIGHT;
+      finscr = sheight / FILECHARHEIGHT;
       if (finscr > flfiles) finscr = flfiles;
 
       pstart = (flstart * sheight) / flfiles;
@@ -563,26 +638,29 @@ void showlscroll(Widget w, caddr_t clientdata, caddr_t calldata)
 /* Button Motion handler for moving the scrollbar up and down		   */
 /*-------------------------------------------------------------------------*/
 
-void draglscroll(Widget w, popupstruct *okaystruct, XButtonEvent *event)
+void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
-   Arg wargs[2];
-   Dimension swidth, sheight;
+   Arg wargs[1];
+   Dimension sheight;
    int phheight, finscr, flsave = flstart;
    short n = 0;
-   Widget filew = okaystruct->filew;
+   xcWidget filew = okaystruct->filew;
 
-   XtnSetArg(XtNwidth, &swidth);
+#ifdef TCL_WRAPPER
+   sheight = Tk_Height(w);
+#else
    XtnSetArg(XtNheight, &sheight);
    XtGetValues(w, wargs, n);
+#endif
 
-   finscr = LISTHEIGHT / FILECHARHEIGHT;
+   finscr = sheight / FILECHARHEIGHT;
    if (finscr > flfiles) finscr = flfiles;
 
    /* center scrollbar on pointer vertical position */   
 
    phheight = (finscr * sheight) / (flfiles * 2);
    flstart = (event->y > phheight) ? ((event->y - phheight) * flfiles) / sheight : 0;
-   if (flstart >= flfiles) flstart = flfiles - 1;
+   if (flstart > (flfiles - finscr + 2)) flstart = (flfiles - finscr + 2);
 
    if (flstart != flsave) {
       showlscroll(w, NULL, NULL);
@@ -594,7 +672,9 @@ void draglscroll(Widget w, popupstruct *okaystruct, XButtonEvent *event)
 /* Set/unset the file filtering function				*/
 /*----------------------------------------------------------------------*/
 
-void setfilefilter(Widget w, popupstruct *okaystruct, caddr_t calldata)
+#ifndef TCL_WRAPPER
+
+void setfilefilter(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
    areastruct.filefilter = (areastruct.filefilter) ? False : True;
 
@@ -603,14 +683,47 @@ void setfilefilter(Widget w, popupstruct *okaystruct, caddr_t calldata)
    newfilelist(okaystruct->filew, okaystruct);
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Generate the file list window					*/
 /*----------------------------------------------------------------------*/
 
-void genfilelist(Widget parent, popupstruct *okaystruct, Dimension width)
+#ifdef TCL_WRAPPER
+
+void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
+{
+   xcWidget 	listarea, lscroll, entertext;
+
+   entertext = okaystruct->textw;
+   listarea = Tk_NameToWindow(xcinterp, ".filelist.listwin.win", parent);
+
+   xcAddEventHandler(listarea, ButtonPressMask, False,
+	  (xcEventHandler)fileselect, okaystruct);
+   xcAddEventHandler(listarea, EnterWindowMask, False,
+	  (xcEventHandler)startfiletrack, NULL);
+   xcAddEventHandler(listarea, LeaveWindowMask, False,
+	  (xcEventHandler)endfiletrack, NULL);
+   flstart = 0;
+   okaystruct->filew = listarea;
+
+   lscroll = Tk_NameToWindow(xcinterp, ".filelist.listwin.sb", parent);
+
+   Tk_CreateEventHandler(lscroll, Button1MotionMask | Button2MotionMask,
+		(Tk_EventProc *)xctk_draglscroll, (ClientData)okaystruct);
+
+   /* force new file list, in case the highlight filter changed */
+
+   if (flistpix != (Pixmap)NULL) XFreePixmap(dpy, flistpix);
+   flistpix = (Pixmap)NULL;
+}
+
+#else
+
+void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 {
    Arg		wargs[8];
-   Widget 	listarea, lscroll, entertext, dofilter;
+   xcWidget 	listarea, lscroll, entertext, dofilter;
    short 	n = 0;
    int		wwidth;
 
@@ -621,15 +734,17 @@ void genfilelist(Widget parent, popupstruct *okaystruct, Dimension width)
    XtnSetArg(XtNfont, appdata.filefont);
 
    entertext = okaystruct->textw;
+
    listarea = XtCreateManagedWidget("Filelist", XwworkSpaceWidgetClass,
 	  parent, wargs, n); n = 0;
    XtAddCallback(listarea, XtNexpose, (XtCallbackProc)listfiles, okaystruct);
-   XtAddEventHandler(listarea, ButtonPressMask, False,
-	  (XtEventHandler)fileselect, okaystruct);
-   XtAddEventHandler(listarea, EnterWindowMask, False,
-	  (XtEventHandler)startfiletrack, NULL);
-   XtAddEventHandler(listarea, LeaveWindowMask, False,
-	  (XtEventHandler)endfiletrack, NULL);
+
+   xcAddEventHandler(listarea, ButtonPressMask, False,
+	  (xcEventHandler)fileselect, okaystruct);
+   xcAddEventHandler(listarea, EnterWindowMask, False,
+	  (xcEventHandler)startfiletrack, NULL);
+   xcAddEventHandler(listarea, LeaveWindowMask, False,
+	  (xcEventHandler)endfiletrack, NULL);
    flstart = 0;
    okaystruct->filew = listarea;
 
@@ -643,8 +758,8 @@ void genfilelist(Widget parent, popupstruct *okaystruct, Dimension width)
 	     parent, wargs, n); n = 0;
 
    XtAddCallback(lscroll, XtNexpose, (XtCallbackProc)showlscroll, NULL);
-   XtAddEventHandler(lscroll, Button1MotionMask | Button2MotionMask,
-	     False, (XtEventHandler)draglscroll, okaystruct);
+   xcAddEventHandler(lscroll, Button1MotionMask | Button2MotionMask,
+	     False, (xcEventHandler)draglscroll, okaystruct);
 
    /* Add a toggle widget to turn file filtering on/off */
 
@@ -667,6 +782,8 @@ void genfilelist(Widget parent, popupstruct *okaystruct, Dimension width)
    flistpix = (Pixmap)NULL;
 }
 
+#endif 	/* TCL_WRAPPER */
+
 /*-------------------------------------------------------------------------*/
 /* Look for a directory name in a string and update cwdname accordingly	   */
 /*-------------------------------------------------------------------------*/
@@ -675,7 +792,7 @@ int lookdirectory(char *lstring)
 {
    int slen;
 
-   tilde_expand(lstring);
+   xc_tilde_expand(lstring);
    slen = strlen(lstring);
 
    if (lstring[slen - 1] == '/' || (opendir(lstring) != NULL)) {
