@@ -6,11 +6,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <ctype.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -19,6 +14,10 @@
 /* Local includes                                                         */
 /*------------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#endif
+
 #include "colordefs.h"
 #include "xcircuit.h"
 
@@ -130,8 +129,8 @@ void loadlgf(int mode)
    /* Done with header. . . okay to clear current page now unless importing */
 
    if (mode == 0) {
-      reset(objectdata, NORMAL);
-      topreset(); 
+      reset(topobject, NORMAL);
+      pagereset(areastruct.page); 
    }
 
    /* Set up filename and object (page) name */
@@ -143,12 +142,12 @@ void loadlgf(int mode)
    /* If the filename has a path component, use only the root */
 
    if ((pdchar = strrchr(inname, '/')) != NULL)
-      sprintf(objectdata->name, "%s", pdchar + 1);
+      sprintf(topobject->name, "%s", pdchar + 1);
    else
-      sprintf(objectdata->name, "%s", inname);
+      sprintf(topobject->name, "%s", inname);
 
    renamepage(areastruct.page);
-   printname(objectdata);
+   printname(topobject);
 
    /* Read objects */
 
@@ -238,8 +237,8 @@ void loadlgf(int mode)
 	       if (tstrp != NULL) {	/* could be a blank line */
 		  stringpart *strptr;
 
-		  NEW_LABEL(newlabel, objectdata);
-	          objectdata->parts++;
+		  NEW_LABEL(newlabel, topobject);
+	          topobject->parts++;
 
 		  labeldefaults(*newlabel, False, xmat(pvalx), ymat(pvaly));
 	          (*newlabel)->justify = TOP | NOTBOTTOM;
@@ -272,8 +271,8 @@ void loadlgf(int mode)
 
 	       /* Allocate wire */
 
-	       NEW_POLY(newwire, objectdata);
-	       objectdata->parts++;
+	       NEW_POLY(newwire, topobject);
+	       topobject->parts++;
 
 	       sscanf(lineptr, "%hd %hd %hd %hd", &pvalx, &pvaly, &pvalx2, &pvaly2);
 	       (*newwire)->number = 2;
@@ -281,6 +280,8 @@ void loadlgf(int mode)
 	       (*newwire)->width = 1.0;
 	       (*newwire)->style = UNCLOSED;
 	       (*newwire)->color = DEFAULTCOLOR;
+	       (*newwire)->num_params = 0;
+	       (*newwire)->passed = NULL;
 	       tmppnts = (*newwire)->points;
 	       tmppnts->x = xmat(pvalx);
 	       tmppnts->y = ymat(pvaly);
@@ -322,14 +323,16 @@ void loadlgf(int mode)
       	          for (lineptr = temp; isspace(*lineptr) && *lineptr != '\n'; lineptr++);
 	       } while (*lineptr == '\n');
 
-	       NEW_POLY(newpoly, objectdata);
-	       objectdata->parts++;
+	       NEW_POLY(newpoly, topobject);
+	       topobject->parts++;
 
 	       (*newpoly)->style = DASHED;
 	       (*newpoly)->color = DEFAULTCOLOR;
 	       (*newpoly)->width = 1.0;
 	       (*newpoly)->number = 4;
                (*newpoly)->points = (pointlist) malloc(4 * sizeof(XPoint));
+	       (*newpoly)->num_params = 0;
+	       (*newpoly)->passed = NULL;
 
                newpoints = (*newpoly)->points;
 	       sscanf(lineptr, "%hd %hd %hd %hd", &pvalx, &pvaly, &pvalx2, &pvaly2);
@@ -383,8 +386,8 @@ void loadlgf(int mode)
 
 	       if (j < xobjs.numlibs || k < xobjs.userlibs[xobjs.numlibs - 1].number) {
 
-		  NEW_OBJINST(newinst, objectdata);
-      	          objectdata->parts++;
+		  NEW_OBJINST(newinst, topobject);
+      	          topobject->parts++;
 
 	          sscanf(lineptr, "%hd %hd %hd %*d %*d %*d %d", &pvalx, &pvaly,
 			&pvalx2, &hval); 
@@ -397,11 +400,17 @@ void loadlgf(int mode)
 		  (*newinst)->scale = 1.0;
 		  (*newinst)->color = DEFAULTCOLOR;
 		  (*newinst)->params = NULL;
+	          (*newinst)->num_params = 0;
+	          (*newinst)->passed = NULL;
 
 		  if (pvalx2 & 0x01) pvalx2 ^= 0x02;
                   if (pvalx2 >= 4) (*newinst)->rotation = -(((pvalx2 - 4) * 90) + 1);
                   else (*newinst)->rotation = (pvalx2 * 90) + 1;
 		  (*newinst)->thisobject = *libobj;
+		  (*newinst)->bbox.lowerleft.x = (*libobj)->bbox.lowerleft.x;
+		  (*newinst)->bbox.lowerleft.y = (*libobj)->bbox.lowerleft.y;
+		  (*newinst)->bbox.width = (*libobj)->bbox.width;
+		  (*newinst)->bbox.height = (*libobj)->bbox.height;
 
 	          /* Add label to "TO" and "FROM" */
 
@@ -416,11 +425,11 @@ void loadlgf(int mode)
 		        if (signets[k] == nval) {
 			   stringpart *strptr;
 
-			   NEW_LABEL(newlabel, objectdata);
-			   objectdata->parts++;
+			   NEW_LABEL(newlabel, topobject);
+			   topobject->parts++;
 			   /* reconnect newinst if displaced by realloc() */
-			   newinst = (objinstptr *)(objectdata->plist
-				+ objectdata->parts - 2);
+			   newinst = (objinstptr *)(topobject->plist
+				+ topobject->parts - 2);
 
 			   labeldefaults(*newlabel, False, (*newinst)->position.x,
 				(*newinst)->position.y);
@@ -428,7 +437,7 @@ void loadlgf(int mode)
 #ifdef SCHEMA
 		  	   if (areastruct.schemon){
 			      (*newlabel)->pin = LOCAL;
-			      (*newlabel)->color = SNAPCOLOR;
+			      (*newlabel)->color = LOCALPINCOLOR;
 		  	   }
 #endif
 			   if (!strcmp(tmpstring, "TO"))
@@ -506,23 +515,23 @@ void loadlgf(int mode)
 
    /* check for unattached labels and delete them */
 
-   for (iolabel = objectdata->plist; iolabel < objectdata->plist +
-	   objectdata->parts; iolabel++)
+   for (iolabel = topobject->plist; iolabel < topobject->plist +
+	   topobject->parts; iolabel++)
       if ((*iolabel)->type == LABEL) {
          if (TOLABEL(iolabel)->rotation == 500) {
 	    genericptr *tmplabel;
 
 	    free(TOLABEL(iolabel)->string);
 	    free(*iolabel);
-            for (tmplabel = iolabel + 1; tmplabel < objectdata->plist +
-                objectdata->parts; tmplabel++) *(tmplabel - 1) = *tmplabel;
-            objectdata->parts--;
+            for (tmplabel = iolabel + 1; tmplabel < topobject->plist +
+                topobject->parts; tmplabel++) *(tmplabel - 1) = *tmplabel;
+            topobject->parts--;
 	    iolabel--;
 	 }
       }
 
-   calcbbox(objectdata);
-   centerview(objectdata);
+   calcbbox(areastruct.topinstance);
+   centerview(areastruct.topinstance);
 
    for (i = 0; i < sigs; i++) free(signals[i]);
    free(signals);
@@ -572,11 +581,11 @@ void loadmat4(caddr_t nullval)
    /* Keep same filename---overwriting file is end-user's own risk */
 
    if ((pdchar = strstr(_STR, ".ps")) != NULL) *pdchar = '\0';
-   sprintf(objectdata->name, "%s", _STR);
+   sprintf(topobject->name, "%s", _STR);
    sprintf(_STR, "Loaded file: %s", inname);
    Wprintf(_STR);
    renamepage(areastruct.page);
-   printname(objectdata);
+   printname(topobject);
 
    /* Create input string buffer */
 
