@@ -10,11 +10,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -1146,7 +1141,7 @@ void updateinstparam(objectptr bobj, int mode)
 
    if (mode == 0) {
       /* pick up calling object and instance from the edit stack */
-      objinstptr pinst = xobjs.stack->thisinst;
+      objinstptr pinst = areastruct.stack->thisinst;
 
       pageobj = pinst->thisobject;
 
@@ -1949,9 +1944,9 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
           case(OBJECT):
 	     if (areastruct.editinplace && stack && (TOOBJINST(areagen)
 			== areastruct.topinstance)) {
-		/* If stack matches xobjs.stack, then don't draw */
+		/* If stack matches areastruct.stack, then don't draw */
 		/* because it would be redundant.		 */
-		pushlistptr alist = *stack, blist = xobjs.stack;
+		pushlistptr alist = *stack, blist = areastruct.stack;
 		while (alist && blist) {
 		   if (alist->thisinst != blist->thisinst) break;
 		   alist = alist->next;
