@@ -1,5 +1,5 @@
 /*-------------------------------------------------------------------------*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -14,10 +14,14 @@
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
 #include "Xw/Form.h"
 #include "Xw/WorkSpace.h"
 #include "Xw/MenuBtn.h"
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -36,15 +40,18 @@
 /* Global Variable definitions						   */
 /*-------------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER
+extern Tcl_Interp *xcinterp;
+#endif
+
 extern Display    *dpy;
-extern Window     win;
 extern GC	  hgc;
 extern Clientdata areastruct;
 extern ApplicationData appdata;
 extern int 	  *appcolors;
-extern Widget     top;
+extern xcWidget     top;
 extern short	  popups;
-extern Widget	  menuwidgets[];
+extern xcWidget	  menuwidgets[];
 extern char *function_names[NUM_FUNCTIONS];
 
 Pixmap   helppix = (Pixmap)NULL;     /* For help window */
@@ -63,7 +70,7 @@ typedef struct {
    char *text;
 } helpstruct;
 
-void printhelppix(Widget w)
+void printhelppix()
 {
    static char *helptitle = "Macro Key Binding Summary:";
    static helpstruct helptext[] = {
@@ -80,6 +87,7 @@ void printhelppix(Widget w)
     { XCF_Double_Snap,	"Double snap-to spacing"},
     { XCF_Halve_Snap,	"Halve snap-to spacing"},
     { XCF_Next_Library,	"Go To Next Library"},
+    { XCF_Library_Pop,	"Return from Library"},
     { XCF_Push,		"Push object"},
     { XCF_Pop,		"Pop object"},
     { XCF_Redraw,	"Refresh screen"},
@@ -140,17 +148,28 @@ void printhelppix(Widget w)
     { XCF_Text_End,	"Go to label end"},
     { XCF_Text_Left,	"Move left one position"},
     { XCF_Text_Right,	"Move right one position"},
+    { XCF_Text_Up,	"Move up one line"},
+    { XCF_Text_Down,	"Move down one line"},
     { XCF_Text_Delete,	"Delete character(s)"},
     { XCF_Text_Return,	"End text edit"},
+    { XCF_Text_Split,	"Split label at cursor"},
     { XCF_SPACER,	NULL},
-    { XCF_Library_Edit,	"Library name edit"},
-    { XCF_Library_Move,	"Library move object/page"},
-    { XCF_Library_Delete, "Library object delete"},
-    { XCF_Library_Hide,	"Library hide object"},
-    { XCF_Library_Duplicate, "Library copy object"},
+    { XCF_Edit_Next,	"Edit: next position"},
+    { XCF_Edit_Break,	"Edit: break line at point"},
+    { XCF_Edit_Delete,	"Edit: delete point"},
+    { XCF_Edit_Insert,	"Edit: insert point"},
+    { XCF_Edit_Param,	"Edit: insert parameter"},
+    { XCF_SPACER,	NULL},
+    { XCF_Library_Edit,	"Library: name edit"},
+    { XCF_Library_Move,	"Library: move object/page"},
+    { XCF_Library_Delete, "Library: object delete"},
+    { XCF_Library_Hide,	"Library: hide object"},
+    { XCF_Library_Duplicate, "Library: copy object"},
+    { XCF_Library_Virtual, "Library: copy instance"},
 #ifdef SCHEMA
     { XCF_SPACER,	NULL},
     { XCF_Pin_Label,	"Make Pin Label"},
+    { XCF_Pin_Global,	"Make Global Pin"},
     { XCF_Info_Label,	"Make Info Label"},
     { XCF_Swap,		"Go to Symbol or Schematic"},
     { XCF_Connectivity,	"See net connectivity"},
@@ -163,19 +182,23 @@ void printhelppix(Widget w)
     };
 
     XGCValues	values;
-    Window hwin = XtWindow(w);
+    Window hwin = DefaultRootWindow(dpy);
     Dimension	htmp, vtmp, lineheight, mwidth;
     int i, j, t1, t2, dum, numlines;
     XCharStruct csdum;
     char *bindings, *bptr, *cptr;
 
+    if (hwin == 0) return;
+
     /* Set up the GC for drawing to the help window pixmap */
 
     if (hgc == NULL) {
        values.foreground = FOREGROUND;
        values.background = BACKGROUND;
+       values.graphics_exposures = False;
        values.font = appdata.helpfont->fid;
-       hgc = XCreateGC(dpy, hwin, GCForeground | GCBackground | GCFont, &values);
+       hgc = XCreateGC(dpy, hwin, GCForeground | GCBackground | GCFont
+		| GCGraphicsExposures, &values);
     }
 
     /* Determine the dimensions of the help text */
@@ -183,7 +206,10 @@ void printhelppix(Widget w)
     mwidth = helpwidth = lineheight = numlines = 0;
 
     for (i = 0; helptext[i].function != XCF_ENDDATA; i++) {
-       if (helptext[i].function == XCF_SPACER) continue;
+       if (helptext[i].function == XCF_SPACER) {
+	  numlines++;
+	  continue;
+       }
 
        htmp = XTextWidth(appdata.helpfont, helptext[i].text, strlen(helptext[i].text));
        if (htmp > mwidth) mwidth = htmp;
@@ -230,7 +256,7 @@ void printhelppix(Widget w)
        return;
     }
     helppix = XCreatePixmap(dpy, hwin, helpwidth, helpheight,
-	   	  DefaultDepthOfScreen(XtScreen(w)));
+	   	  DefaultDepthOfScreen(DefaultScreenOfDisplay(dpy)));
 
     XSetForeground(dpy, hgc, FOREGROUND);
     XFillRectangle(dpy, helppix, hgc, 0, 0, helpwidth, helpheight);
@@ -240,7 +266,10 @@ void printhelppix(Widget w)
 	helptitle, strlen(helptitle))) >> 1, t1 + 2, helptitle, strlen(helptitle));
     vtmp = lineheight + 15;
     for (i = 0; helptext[i].function != XCF_ENDDATA; i++) {
-       if (helptext[i].function == XCF_SPACER) continue;
+       if (helptext[i].function == XCF_SPACER) {
+	  vtmp += lineheight;
+	  continue;
+       }
        XDrawString(dpy, helppix, hgc, 7, vtmp, helptext[i].text,
 		strlen(helptext[i].text));
        bindings = binding_to_string(-1, helptext[i].function);
@@ -263,14 +292,32 @@ void printhelppix(Widget w)
     XDrawLine(dpy, helppix, hgc, 0, t1 + 7, helpwidth, t1 + 7);
 }
 
-/*-----------------------------------------*/
-/* Create the help popup window		   */
-/*-----------------------------------------*/
+#ifdef TCL_WRAPPER
 
-void starthelp(Widget button, caddr_t clientdata, caddr_t calldata)
+/*----------------------------------------------*/
+/* Map the help popup window	(Xt version)	*/
+/*----------------------------------------------*/
+
+void starthelp(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
-   Arg		wargs[8];
-   Widget	popup, cancelbutton, hspace, help2;
+   Tk_Window popup, hspace;
+
+   Tcl_Eval(xcinterp, "wm deiconify .help");
+   hspace = Tk_NameToWindow(xcinterp, ".help.listwin.win", areastruct.area);
+   if (Tk_IsMapped(hspace))
+      exposehelp(hspace, NULL, NULL);
+}
+
+#else
+
+/*----------------------------------------------*/
+/* Create the help popup window	(Xt version)	*/
+/*----------------------------------------------*/
+
+void starthelp(xcWidget button, caddr_t clientdata, caddr_t calldata)
+{
+   Arg		wargs[11];
+   xcWidget	popup, cancelbutton, hspace, help2, hsb;
    short 	n = 0;
    popupstruct  *okaystruct;
    buttonsave   *savebutton = (buttonsave *)malloc(sizeof(buttonsave));
@@ -303,11 +350,11 @@ void starthelp(Widget button, caddr_t clientdata, caddr_t calldata)
 
    /* Generate the pixmap and write the help text to it */
 
-   if (helppix == (Pixmap)NULL) printhelppix(button);
+   if (helppix == (Pixmap)NULL) printhelppix();
 
    /* Use the pixmap size to size the help window */
 
-   if (xpos + helpwidth > xmax)  xpos = xmax - helpwidth  - 4;
+   if (xpos + helpwidth + SBARSIZE > xmax)  xpos = xmax - helpwidth - SBARSIZE - 4;
    if (ypos + helpheight > ymax) ypos = ymax - helpheight - 4;
    if (ypos < 4) ypos = 4;
 
@@ -319,7 +366,6 @@ void starthelp(Widget button, caddr_t clientdata, caddr_t calldata)
    help_up = True;
    helptop = 0;
 
-   XtnSetArg(XtNwidth, helpwidth);
    XtnSetArg(XtNyResizable, True);
    XtnSetArg(XtNxResizable, False);
    help2 = XtCreateManagedWidget("help2", XwformWidgetClass,
@@ -329,16 +375,29 @@ void starthelp(Widget button, caddr_t clientdata, caddr_t calldata)
    cancelbutton = XtCreateManagedWidget("Dismiss", XwmenuButtonWidgetClass,
 	help2, wargs, n); n = 0;
 
+   XtnSetArg(XtNwidth, helpwidth);
+   XtnSetArg(XtNheight, areastruct.height);
    XtnSetArg(XtNyRefWidget, cancelbutton);
    XtnSetArg(XtNyAddHeight, True);
    XtnSetArg(XtNyAttachBottom, True);
+   XtnSetArg(XtNyResizable, True);
    XtnSetArg(XtNborderWidth, 0);
-   XtnSetArg(XtNheight, helpheight);
-   XtnSetArg(XtNwidth, helpwidth);
-   XtnSetArg(XtNxAttachRight, True);
+   XtnSetArg(XtNxAttachRight, False);
    hspace = XtCreateManagedWidget("HSpace", XwworkSpaceWidgetClass,
 	help2, wargs, n); n = 0;
 
+   /* Create scrollbar */
+   XtnSetArg(XtNwidth, SBARSIZE);
+   XtnSetArg(XtNxRefWidget, hspace);
+   XtnSetArg(XtNxAddWidth, True);
+   XtnSetArg(XtNyRefWidget, cancelbutton);
+   XtnSetArg(XtNyAddHeight, True);
+   XtnSetArg(XtNyResizable, True);
+   XtnSetArg(XtNyAttachBottom, True);
+   XtnSetArg(XtNborderWidth, 1);
+   hsb = XtCreateManagedWidget("HSB", XwworkSpaceWidgetClass,
+		help2, wargs, n); n = 0;
+
    okaystruct = (popupstruct *) malloc(sizeof(popupstruct));
    okaystruct->buttonptr = savebutton;
    okaystruct->popup = popup;
@@ -353,61 +412,101 @@ void starthelp(Widget button, caddr_t clientdata, caddr_t calldata)
    XtnSetArg(XtNx, ((helpwidth - bwidth) >> 1));
    XtSetValues(cancelbutton, wargs, n); n = 0;
 
-   /* Check for total size of window compared to computer screen */
-   /* and add simple button-click callback if necessary.	 */
-
    XtSetArg(wargs[0], XtNheight, &pheight);
    XtGetValues(help2, wargs, 1);
 
    if (pheight > (ymax - 8)) {
-      XtnSetArg(XtNheight, helpheight - (pheight - ymax) - 8);
-      XtSetValues(hspace, wargs, n); n = 0;
       XtnSetArg(XtNheight, ymax - 8);
-      XtSetValues(help2, wargs, n);
-      XtSetValues(popup, wargs, n); n = 0;
-      XtAddEventHandler(hspace, ButtonMotionMask, False,
-		(XtEventHandler)simplescroll, NULL);
+      XtSetValues(help2, wargs, n); n = 0;
    }
 
+   XtAddEventHandler(hsb, ButtonMotionMask | ButtonPressMask, False,
+		(XtEventHandler)simplescroll, hspace);
+
    /* Expose and End callbacks */
 
    XtAddCallback(cancelbutton, XtNselect, (XtCallbackProc)destroypopup, okaystruct); 
-   XtAddCallback(hspace, XtNexpose, (XtCallbackProc)printhelp, popup);
+   XtAddCallback(hspace, XtNexpose, (XtCallbackProc)exposehelp, NULL);
+   XtAddCallback(hsb, XtNexpose, (XtCallbackProc)showhsb, NULL);
 }
 
+#endif
+
 /*----------------------------------------------*/
 /* Very simple scroll mechanism	 (grab-and-pan)	*/
 /*----------------------------------------------*/
 
-void simplescroll(Widget hspace, caddr_t clientdata, XMotionEvent *event)
+void simplescroll(xcWidget hsb, xcWidget hspace, XMotionEvent *event)
 {
-   Window hwin = XtWindow(hspace);
+   Dimension oldtop = helptop;
 
-   if (event->y != areastruct.save.y) {
-      helptop += (int)(event->y - areastruct.save.y);
-      if (helptop < 0) helptop = 0;
-      else if (helptop > helpheight - hheight) helptop = helpheight - hheight;
+   helptop = (((int)(event->y) * helpheight) / hheight) - (hheight / 2);
+
+   if (helptop < 0) helptop = 0;
+   else if (helptop > helpheight - hheight) helptop = helpheight - hheight;
+
+   if (helptop != oldtop) {
+      showhsb(hsb, NULL, NULL);
+      printhelp(hspace);
    }
-   areastruct.save.y = event->y;
+}
+ 
+/*----------------------------------------------*/
+/* Expose callback for the help scrollbar	*/
+/*----------------------------------------------*/
 
-   /* Draw the pixmap to the window */
+void showhsb(xcWidget hsb, caddr_t clientdata, caddr_t calldata)
+{
+   Window hwin = xcWindow(hsb);
+   Dimension sheight;
+   int pstart, pheight;
+   short n = 0;
 
-   XCopyArea(dpy, helppix, hwin, hgc, 0, helptop - 5, helpwidth, helpheight,
-	0, 0);
+   if (helppix == (Pixmap)NULL) printhelppix();
+   if (helpheight == 0) helpheight = 1;
+
+   pstart = (helptop * hheight) / helpheight;
+   pheight = (hheight * hheight) / helpheight;
+
+   if (pheight < 3) pheight = 3;
+
+   XClearArea(dpy, hwin, 0, 0, SBARSIZE, pstart, False);
+   XClearArea(dpy, hwin, 0, pstart + pheight, SBARSIZE,
+		hheight - (pstart + pheight), False);
+
+   XSetForeground(dpy, hgc, BARCOLOR);
+   XFillRectangle(dpy, hwin, hgc, 0, pstart, SBARSIZE, pheight);
 }
 
 /*----------------------------------------------*/
 /* Expose callback for the help window		*/
 /*----------------------------------------------*/
 
-void printhelp(Widget hspace, caddr_t clientdata, caddr_t calldata)
+void exposehelp(xcWidget hspace, caddr_t clientdata, caddr_t calldata)
 {
    Arg wargs[1];
-   Window hwin = XtWindow(hspace);
-   XEvent discard;
 
+#ifdef TCL_WRAPPER
+   hheight = Tk_Height(hspace);
+#else
    XtSetArg(wargs[0], XtNheight, &hheight);
    XtGetValues(hspace, wargs, 1);
+#endif
+
+   if (helppix == (Pixmap)NULL) printhelppix();
+   if (hheight < 1) hheight = 1;
+
+   printhelp(hspace);
+}
+
+/*----------------------------------------------*/
+/* Expose callback for the help window		*/
+/*----------------------------------------------*/
+
+void printhelp(xcWidget hspace)
+{
+   Window hwin = xcWindow(hspace);
+   XEvent discard;
 
    /* Draw the pixmap to the window */
 
