@@ -11,11 +11,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>	 /* for tolower(), toupper() */
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -84,17 +79,19 @@ static char *function_names[NUM_FUNCTIONS] = {
    "Page Directory ", "Library Pop",
    "Help", "Redraw", "View", "Zoom In", "Zoom Out", "Pan",
    "Double Snap", "Halve Snap", "Pan Left", "Pan Right", "Pan Up",
-   "Pan Down", "Write", "Rotate", "Flip X", "Flip Y", "Snap",
+   "Pan Down", "Write", "Rotate", "Flip X", "Flip Y", "Snap", "Snap To",
    "Pop", "Push", "Delete", "Select", "Box", "Arc", "Text",
    "Exchange", "Copy", "Join", "Unjoin", "Spline", "Edit",
    "Undelete", "Select Save", "Unselect", "Dashed", "Dotted",
    "Solid", "Prompt", "Dot", "Wire", "Start", "Finish", "Cancel",
-   "Nothing", "Exit"
+   "Nothing", "Exit",
 
 #ifdef SCHEMA
-   , "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label",
-   "Connectivity", "Sim", "SPICE", "PCB", "SPICE Flat"
+   "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label",
+   "Connectivity", "Sim", "SPICE", "PCB", "SPICE Flat",
 #endif
+
+   NULL			/* sentinel */
 };
 
 /*--------------------------------------------------------------*/
@@ -186,8 +183,14 @@ int string_to_func(const char *funcstring, short *value)
    int i;
 
    for (i = 0; i < NUM_FUNCTIONS; i++)
+   {
+      if (function_names[i] == NULL) {
+	 Fprintf(stderr, "Error: resolve bindings and function strings!\n");
+	 return -1;	  /* should not happen? */
+      }
       if (!strcmp(funcstring, function_names[i]))
 	 return i;
+   }
 
    /* Check if this string might have a value attached */
 
@@ -636,6 +639,7 @@ void default_keybindings()
    add_binding(SHIFT | BUTTON1, XCF_Finish);
    add_binding(BUTTON2, XCF_Finish);
    add_binding(BUTTON3, XCF_Cancel);
+   add_binding(ALT | XK_s, XCF_SnapTo);
 
 #ifdef SCHEMA
    add_binding(ALT | XK_q, XCF_Netlist);
@@ -644,8 +648,8 @@ void default_keybindings()
    add_binding(XK_G, XCF_Pin_Global);
    add_binding(XK_I, XCF_Info_Label);
    add_binding(ALT | XK_w, XCF_Connectivity);
-   add_binding(ALT | XK_s, XCF_Sim);
-   add_binding(SHIFT | ALT | XK_S, XCF_SPICE);
+   add_binding(ALT | XK_d, XCF_Sim);
+   add_binding(ALT | XK_a, XCF_SPICE);
    add_binding(ALT | XK_f, XCF_SPICEflat);
    add_binding(ALT | XK_p, XCF_PCB);
 #endif
