@@ -38,6 +38,7 @@ option add *output*font -*-times-bold-r-normal--14-*
 option add *output*txt*font -*-courier-bold-r-normal--14-*
 
 option add *output.textent.butp.psmenu*background beige
+option add *output.textent.txtf.sizb.sizemenu*background beige
 option add *output.textent.buto.orientmenu*background beige
 
 option add *help*activeBackground brown4
@@ -58,9 +59,8 @@ option add *dialog*txt*font -*-courier-bold-r-normal--14-*
 # Main xcircuit drawing window
 #-------------------------------
 
-toplevel .xcircuit -width 1000 -height 785 -background red
+toplevel .xcircuit -width 1000 -height 800 -background red
 wm title .xcircuit XCircuit
-# tk_setPalette red
 
 # This gets rid of the original "wish", in favor of our own window
 
@@ -78,7 +78,7 @@ pack propagate .xcircuit false
 
 pack .xcircuit.menubar -side top -fill x -pady 1
 pack .xcircuit.mainframe -side top -expand true -fill both
-pack .xcircuit.infobar -side top -fill x -pady 1
+pack .xcircuit.infobar -side bottom -fill x -pady 1
 
 frame .xcircuit.mainframe.mainarea -background red
 frame .xcircuit.mainframe.toolbar -bg beige
@@ -123,19 +123,19 @@ pack .xcircuit.menubar.filebutton .xcircuit.menubar.editbutton \
 
 # The bottom message bar
 
-tklabel .xcircuit.menubar.message -text "Welcome to Xcircuit v3.1" \
+label .xcircuit.menubar.message -text "Welcome to Xcircuit v3.1" \
 	-background beige -justify left -anchor w
 pack .xcircuit.menubar.message -side top -fill x -padx 2 -ipady 3 -pady 2
 
 button .xcircuit.infobar.symb -text "Symbol" -bg gray30 -fg white
 button .xcircuit.infobar.schem -text "Schematic" -bg red -fg white
-tklabel .xcircuit.infobar.message1 -text "Editing: Page 1" \
+label .xcircuit.infobar.message1 -text "Editing: Page 1" \
 	-background beige
-tklabel .xcircuit.infobar.message2 -text "Grid 1/6 in : Snap 1/12 in" \
+label .xcircuit.infobar.message2 -text "Grid 1/6 in : Snap 1/12 in" \
 	-background beige -justify left -anchor w
 pack .xcircuit.infobar.symb .xcircuit.infobar.schem \
-	.xcircuit.infobar.message1 -side left -padx 2 -ipadx 6
-pack .xcircuit.infobar.message2 -side bottom -padx 2 -ipadx 6 -fill x
+	.xcircuit.infobar.message1 -side left -padx 2 -ipadx 6 -fill y
+pack .xcircuit.infobar.message2 -padx 2 -ipadx 6 -expand true -fill both
 
 #----------------------------------------------------------------------
 # Support procedures for tag callbacks
@@ -150,19 +150,40 @@ proc xcircuit::setpstype {mode} {
       {eps} { .output.textent.butp configure -text "Embedded (EPS)"
 	 grid remove .output.textent.but7
 	 grid remove .output.textent.butf
-	 grid remove .output.textent.txtp
+	 grid remove .output.textent.txtf
       }
       {full} {.output.textent.butp configure -text "Full Page"
 	 grid .output.textent.but7 -row 6 -column 3 -pady 5 -ipadx 10
 	 grid .output.textent.butf -row 5 -column 2 -padx 10
-	 grid .output.textent.txtp -row 6 -column 2 -sticky ew -padx 10
+	 grid .output.textent.txtf -row 6 -column 2 -sticky ew -padx 10
       }
    }
    ::set pstype $mode
+   xcircuit::page encapsulation $mode
+}
+
+proc xcircuit::dolinks {} {
+   global imultiple dmultiple
+   set ilinks [xcircuit::page links independent]
+   if {$ilinks > 1} {
+      set imultiple 1
+   } else {
+      set imultiple 0
+   }
+   if {$ilinks == 1} { set plural ""} else { set plural "s"}
+   .output.title.imulti configure -text "$ilinks schematic$plural"
+
+   if {$dmultiple == 1} {
+      set dlinks [xcircuit::page links dependent]
+   } else {
+      set dlinks 0
+   }
+   if {$dlinks == 1} { set plural ""} else { set plural "s"}
+   .output.title.dmulti configure -text "$dlinks subcircuit$plural"
 }
 
 proc xcircuit::pageupdate { {subcommand "none"} } {
-   global tcl_precision autofit orient multiple
+   global tcl_precision autofit orient imultiple dmultiple
    if {[info level] <= 1} {
      switch -- $subcommand {
 	save {
@@ -193,10 +214,14 @@ proc xcircuit::pageupdate { {subcommand "none"} } {
 	  .output.textent.txt4 delete 0 end
 	  ::set tcl_precision 3
 	  .output.textent.txt4 insert 0 [xcircuit::page width]
+	  .output.textent.txt4 insert end " "
+	  .output.textent.txt4 insert end [xcircuit::coordstyle get]
 	  .output.textent.txt5 delete 0 end
 	  .output.textent.txt5 insert 0 [xcircuit::page height]
-	  .output.textent.txtp delete 0 end
-	  .output.textent.txtp insert 0 [xcircuit::page size]
+	  .output.textent.txt5 insert end " "
+	  .output.textent.txt5 insert end [xcircuit::coordstyle get]
+	  .output.textent.txtf.txtp delete 0 end
+	  .output.textent.txtf.txtp insert 0 [xcircuit::page size]
 	  ::set tcl_precision $oldp
 	  xcircuit::setpstype [xcircuit::page encapsulation]
 	  ::set orient [xcircuit::page orientation]
@@ -205,15 +230,8 @@ proc xcircuit::pageupdate { {subcommand "none"} } {
 	  } else {
 	     .output.textent.buto configure -text Landscape
 	  }
-	  ::set links [xcircuit::page links]
-	  if {$links > 1} {
-	    set multiple 1
-	    .output.title.multi configure -text "$links pages"
-	    pack .output.title.multi -side right
-	  } else {
-	    set multiple 0
-	    pack forget .output.title.multi
-	  }
+	  xcircuit::dolinks
+
 	  ::set autofit [xcircuit::page fit]
 	  if {[string match *.* $fname] == 0} {append fname .ps}
 	  if {[glob -nocomplain ${fname}] == {}} {
@@ -223,7 +241,8 @@ proc xcircuit::pageupdate { {subcommand "none"} } {
 	  }
 	  .output.bbar.okay configure -command \
 		{xcircuit::page filename [.output.textent.txt1 get]; \
-		 xcircuit::page save; wm withdraw .output}
+		 if {$dmultiple == 1} {xcircuit::page save} else { \
+		 xcircuit::page saveonly }; wm withdraw .output}
         }
      }
   }
@@ -291,6 +310,62 @@ proc xcircuit::setsymschem {} {
    }
 }
    
+#----------------------------------------------------------------------
+# Set the coordinate style to inches from cm and vice versa.
+# This routine avoids switching from fractional to decimal inches
+# and vice versa if we are already in one of the two inches modes.
+#
+# with no argument, or argument "get", returns the "short" name
+# ("cm" or "in") of the style.
+#----------------------------------------------------------------------
+
+proc xcircuit::coordstyle { { mode get } } {
+   set curstyle [xcircuit::config coordstyle]
+   switch -- $mode {
+      inches {
+	 switch -- $curstyle {
+	    centimeters {
+	       xcircuit::config coordstyle "decimal inches"
+	       xcircuit::pageupdate
+	    }
+	 }
+      }
+      centimeters -
+      cm {
+	 switch -- $curstyle {
+	    centimeters {
+	    }
+	    default {
+	       xcircuit::config coordstyle "centimeters"
+	       xcircuit::pageupdate
+	    }
+	 }
+      }
+      get {
+	 switch -- $curstyle {
+	    centimeters {
+	       return "cm"
+	    }
+	    default {
+	       return "in"
+	    }
+	 }
+      }
+   }
+}
+
+proc xcircuit::raiseconsole {} {
+   xcircuit::consoleup
+   .xcircuit.menubar.filebutton.filemenu entryconfigure \
+	 4 -label "No Console" -command {xcircuit::lowerconsole}
+}
+
+proc xcircuit::lowerconsole {} {
+   xcircuit::consoledown
+   .xcircuit.menubar.filebutton.filemenu entryconfigure \
+	 4 -label "Tcl Console" -command {xcircuit::raiseconsole}
+}
+
 #----------------------------------------------------------------------
 # Command tags---these let the command-line entry functions update the
 # Tk windows, so that the Tk window structure doesn't need to be hard-
@@ -343,8 +418,8 @@ grid rowconfigure .filelist.listwin 0 -weight 1 -minsize 100
 frame .filelist.textent.title -bg beige
 pack .filelist.textent.title -side top -fill x
 
-tklabel .filelist.textent.title.field -text "Select file to load:" -bg beige
-tklabel .filelist.textent.title.chklabel -text "Filter" -bg beige
+label .filelist.textent.title.field -text "Select file to load:" -bg beige
+label .filelist.textent.title.chklabel -text "Filter" -bg beige
 checkbutton .filelist.textent.title.filter -bg beige -variable filter \
    -command {event generate .filelist.listwin.win <ButtonPress> -button 3 ; \
 	event generate .filelist.listwin.win <ButtonRelease> -button 3}
@@ -373,7 +448,8 @@ bind .filelist.textent.txt <Return> \
 
 # First, set the variables associated with toggle and radio buttons
 set autofit 0
-set multiple 1
+set imultiple 0
+set dmultiple 1
 
 toplevel .output -bg beige
 wm withdraw .output
@@ -386,20 +462,27 @@ pack .output.title -side top -padx 20 -pady 7 -fill x
 pack .output.textent -side top -padx 20 -pady 7 -fill x
 pack .output.bbar -side top -padx 20 -pady 7 -fill x
 
-tklabel .output.title.field -text "PostScript output properties (Page 1):" -bg tan
-checkbutton .output.title.multi -text "1 page" -bg beige -variable multiple \
-	-command {.output.textent.txt1 delete 0 end; .output.textent.but1 configure \
-	-text Apply; xcircuit::page filename {}; focus .output.textent.txt1}
+label .output.title.field -text "PostScript output properties (Page 1):" -bg tan
+checkbutton .output.title.imulti -text "1 schematic" -bg beige -variable imultiple \
+	-command {xcircuit::dolinks ; if {$imultiple == 1} {.output.textent.txt1 \
+	delete 0 end; .output.textent.but1 configure -text Apply; xcircuit::page \
+	filename {}; focus .output.textent.txt1 ; xcircuit::dolinks }}
+checkbutton .output.title.dmulti -text "0 subcircuits" -bg beige -variable dmultiple \
+	-command {xcircuit::dolinks ; .output.textent.txt1 \
+	delete 0 end; .output.textent.but1 configure -text Apply; xcircuit::page \
+	filename {}; focus .output.textent.txt1 ; xcircuit::dolinks }
+pack .output.title.dmulti -side right -padx 5
+pack .output.title.imulti -side right -padx 5
 
 pack .output.title.field -side left
 
-tklabel .output.textent.lab1 -text "Filename:" -bg beige
-tklabel .output.textent.lab2 -text "Page label:" -bg beige
-tklabel .output.textent.lab3 -text "Scale:" -bg beige
-tklabel .output.textent.lab4 -text "X Size (in):" -bg beige
-tklabel .output.textent.lab5 -text "Y Size (in):" -bg beige
-tklabel .output.textent.lab6 -text "Orientation:" -bg beige
-tklabel .output.textent.lab7 -text "Mode:" -bg beige
+label .output.textent.lab1 -text "Filename:" -bg beige
+label .output.textent.lab2 -text "Page label:" -bg beige
+label .output.textent.lab3 -text "Scale:" -bg beige
+label .output.textent.lab4 -text "Width:" -bg beige
+label .output.textent.lab5 -text "Height:" -bg beige
+label .output.textent.lab6 -text "Orientation:" -bg beige
+label .output.textent.lab7 -text "Mode:" -bg beige
 
 entry .output.textent.txt1 -bg white -relief sunken -width 20
 entry .output.textent.txt2 -bg white -relief sunken -width 20
@@ -414,7 +497,13 @@ menubutton .output.textent.butp -text "Embedded (EPS)" -bg beige \
 
 checkbutton .output.textent.butf -text "Auto-fit" -bg beige -variable autofit \
 	-onvalue true -offvalue false -command {xcircuit::page fit $autofit}
-entry .output.textent.txtp -bg white -relief sunken -width 14
+frame .output.textent.txtf -bg beige
+menubutton .output.textent.txtf.sizb -text "Sizes" -bg beige \
+	-menu .output.textent.txtf.sizb.sizemenu
+entry .output.textent.txtf.txtp -bg white -relief sunken -width 14
+
+pack .output.textent.txtf.txtp -side left -fill y
+pack .output.textent.txtf.sizb -side left
 
 button .output.textent.but1 -text Apply -bg beige \
 	-command {xcircuit::page filename [.output.textent.txt1 get];\
@@ -432,7 +521,7 @@ button .output.textent.but5 -text Apply -bg beige \
 	-command {xcircuit::page height [.output.textent.txt5 get];\
 	.output.textent.but5 configure -text Okay}
 button .output.textent.but7 -text Apply -bg beige \
-	-command {xcircuit::page size [.output.textent.txt7 get];\
+	-command {xcircuit::page size [.output.textent.txtf.txtp get];\
 	.output.textent.but7 configure -text Okay}
 
 bind .output.textent.txt1 <Return> {.output.textent.but1 invoke}
@@ -489,6 +578,42 @@ $m add radio -label "Full Page" -variable pstype -value full -command \
 pack .output.bbar.okay -side left -ipadx 10
 pack .output.bbar.cancel -side right -ipadx 10
 
+set m [menu .output.textent.txtf.sizb.sizemenu -tearoff 0]
+$m add radio -label "Letter (ANSI A)" -variable sheetsize -value letter -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "8.5 x 11.0 in"}
+$m add radio -label "Legal" -variable sheetsize -value legal -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "8.5 x 14.0 in"}
+$m add radio -label "Statement" -variable sheetsize -value statement -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "5.5 x 8.5 in"}
+$m add radio -label "Tabloid (ANSI B)" -variable sheetsize -value tabloid -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "11.0 x 17.0 in"}
+$m add radio -label "Ledger" -variable sheetsize -value ledger -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "17.0 x 11.0 in"}
+$m add radio -label "Folio" -variable sheetsize -value folio -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "8.5 x 13.0 in"}
+$m add radio -label "Quarto" -variable sheetsize -value quarto -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "8.472 x 10.833 in"}
+$m add radio -label "10x14" -variable sheetsize -value tenfourteen -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "10.0 x 14.0 in"}
+$m add radio -label "Executive" -variable sheetsize -value executive -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "7.5 x 10.0 in"}
+$m add radio -label "ANSI C" -variable sheetsize -value ansic -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "17.0 x 22.0 in"}
+$m add radio -label "ANSI D" -variable sheetsize -value ansid -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "22.0 x 34.0 in"}
+$m add radio -label "ANSI E" -variable sheetsize -value ansie -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "34.0 x 44.0 in"}
+$m add radio -label "A3" -variable sheetsize -value a3 -command \
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "29.7 x 42.0 cm"}
+$m add radio -label "A4" -variable sheetsize -value a4 -command \
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "21.0 x 29.7 cm"}
+$m add radio -label "A5" -variable sheetsize -value a5 -command \
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "14.82 x 18.43 cm"}
+$m add radio -label "B4" -variable sheetsize -value b4 -command \
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "25.7 x 36.4 cm"}
+$m add radio -label "B5" -variable sheetsize -value b5 -command \
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "18.2 x 25.7 cm"}
+
 #--------------------------------------
 # Create the popup help window
 #--------------------------------------
@@ -502,7 +627,7 @@ frame .help.listwin
 pack .help.title -side top -fill x
 pack .help.listwin -side top -fill both -expand true
 
-tklabel .help.title.field -text "XCircuit Help" -bg beige
+label .help.title.field -text "XCircuit Help" -bg beige
 button .help.title.dbut -text "Dismiss" -bg beige -command {wm withdraw .help}
 pack .help.title.field -side left -padx 10
 pack .help.title.dbut -side right -ipadx 10
@@ -547,7 +672,7 @@ entry .dialog.textent.txt -bg white -relief sunken -width 50
 pack .dialog.textent.title -side top -fill x
 pack .dialog.textent.txt -side bottom -fill x -expand true
 
-tklabel .dialog.textent.title.field -text "Select file to load:" -bg beige
+label .dialog.textent.title.field -text "Select file to load:" -bg beige
 pack .dialog.textent.title.field -side left
 
 button .dialog.bbar.okay -text Okay -bg beige
@@ -571,7 +696,7 @@ frame .query.bbar	-bg beige
 pack .query.title -side top -padx 20 -pady 7 -fill x
 pack .query.bbar -side top -padx 20 -pady 7 -fill x
 
-tklabel .query.title.field -text "Select:" -bg beige
+label .query.title.field -text "Select:" -bg beige
 pack .query.title.field -side left
 
 button .query.bbar.okay -text Okay -bg beige
@@ -747,9 +872,15 @@ proc xcircuit::promptloadlibrary {} {
 
 proc xcircuit::promptsavelib { libno } {
    .dialog.bbar.okay configure -command \
-	{xcircuit::library $libno save [.dialog.textent.txt get];\
-	 wm withdraw .dialog}
+	[subst {xcircuit::library $libno save \[.dialog.textent.txt get\];\
+	 wm withdraw .dialog}]
    .dialog.textent.title.field configure -text "Enter name for library:"
+   .dialog.textent.txt delete 0 end
+   set lname [xcircuit::library $libno]
+   if {[string first Library: $lname] == 0} \
+      {set lname [string range $lname 8 end]}
+   set lname [string trimleft $lname]
+   .dialog.textent.txt insert 0 [string map {" " _} $lname]
    wm deiconify .dialog
    focus .dialog.textent.txt
 }
@@ -924,6 +1055,101 @@ proc xcircuit::xschema {value} {
    }
 }
 
+set tools [list b a s t cp e d2 cw ccw fx fy pu2 po2 mk pz uj co bd fi \
+	pm li yp pl z4 z5 i]
+
+proc xcircuit::createtoolbar {} {
+   global XCIRCUIT_LIB_DIR
+   global tools
+   set tooltips [list "draw box" "draw arc" "draw spline" "enter text" \
+	"copy element" "edit element" "delete element" \
+	"rotate 15 degrees clockwise" "rotate 15 degrees counterclockwise" \
+	"flip horizontal" "flip vertical" "push (edit object)" \
+	"pop (return from object edit)" "make an object from selection" \
+	"join elements into polygon or path" "separate path into elements" \
+	"set color" "set border and line properties" "set fill properties" \
+	"parameterize properties" "go to next library" "go to library directory" \
+	"go to page directory" "zoom in" "zoom out" "pop up help window"]	
+   set toolactions [list {xcircuit::polygon make box} {xcircuit::arc make} \
+	{xcircuit::spline make} {xcircuit::label make} {xcircuit::copy selected} \
+	{xcircuit::edit selected} {xcircuit::delete selected} {xcircuit::rotate 15} \
+	{xcircuit::rotate -15} {xcircuit::flip horizontal} \
+	{xcircuit::flip vertical} {xcircuit::push selected} {xcircuit::pop} \
+	{xcircuit::promptmakeobject} {xcircuit::path join selected} \
+	{xcircuit::path unjoin selected} \
+	{tk_popup .colormenu [expr {[winfo rootx \
+		.xcircuit.mainframe.toolbar.bco] 
+		- [winfo width .colormenu]}] \
+		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bco] \
+		- [winfo height .colormenu] / 2}] } \
+	{tk_popup .bordermenu [expr {[winfo rootx \
+		.xcircuit.mainframe.toolbar.bbd] \
+		- [winfo width .bordermenu]}] \
+		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bbd] \
+		- [winfo height .bordermenu] / 2}] } \
+	{tk_popup .fillmenu [expr {[winfo rootx \
+		.xcircuit.mainframe.toolbar.bfi] \
+		- [winfo width .fillmenu]}] \
+		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bfi] \
+		- [winfo height .fillmenu] / 2}] } \
+	{tk_popup .parammenu [expr {[winfo rootx \
+		.xcircuit.mainframe.toolbar.bpm] \
+		- [winfo width .parammenu]}] \
+		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bpm] \
+		- [winfo height .parammenu] / 2}] } \
+	{xcircuit::library next} \
+	{xcircuit::library directory} {xcircuit::page directory} \
+	{xcircuit::zoom 1.5; refresh} {xcircuit::zoom [expr {1 / 1.5}]; refresh} \
+	{wm deiconify .help} ]
+
+   for {set i 0} {$i < [llength $tools]} {incr i 1} {
+      set bname [lindex $tools $i]
+      set btip [lindex $tooltips $i]
+      image create xpm img_${bname} -file ${XCIRCUIT_LIB_DIR}/pixmaps/${bname}.xpm
+      button .xcircuit.mainframe.toolbar.b${bname} -image img_${bname} \
+		-command [lindex $toolactions $i]
+      bind .xcircuit.mainframe.toolbar.b${bname} <Enter> \
+		[subst {.xcircuit.infobar.message2 configure -text "$btip"}]
+      bind .xcircuit.mainframe.toolbar.b${bname} <Leave> \
+		[subst {.xcircuit.infobar.message2 configure -text ""}]
+   }
+ 
+   # pack the first button so we can query its height for arrangement.
+   # this assumes that the height of each button is the same!
+   set bname [lindex $tools 0]
+   place .xcircuit.mainframe.toolbar.b${bname} -x 0 -y 0
+   update idletasks
+}
+
+proc xcircuit::arrangetoolbar {} {
+   global tools
+
+   set numtools [llength $tools]
+   for {set i 0} {$i < $numtools} {incr i 1} {
+      set bname [lindex $tools $i]
+      place forget .xcircuit.mainframe.toolbar.b${bname}
+   }
+   set bname [lindex $tools 0]
+   set bheight [winfo height .xcircuit.mainframe.toolbar.b${bname}]
+   set bwidth [winfo width .xcircuit.mainframe.toolbar.b${bname}]
+   set wheight [winfo height .xcircuit.mainframe]
+   set nrows [expr {$wheight / $bheight}]
+   .xcircuit.mainframe.toolbar configure -width [expr {$bwidth}]
+   set j 0
+   set k 0
+   for {set i 0} {$i < [llength $tools]} {incr i; incr j} {
+      if {$j == $nrows} {
+	 set j 0
+	 incr k
+	 .xcircuit.mainframe.toolbar configure -width [expr {($k + 1) * $bwidth}]
+      }
+      set bname [lindex $tools $i]
+      place .xcircuit.mainframe.toolbar.b${bname} \
+		-x [expr {$k * $bwidth}] \
+		-y [expr {$j * $bheight}]
+   }
+}
+
 proc xcircuit::toolbar {value} {
    switch -- $value {
       true -
@@ -974,7 +1200,7 @@ set bbox invisible
 set editinplace true
 set pinpositions invisible
 set centerobject true
-set manhattandraw true
+set manhattandraw false
 set polyclosed true
 set bboxtype false
 set substringparam false
@@ -994,17 +1220,21 @@ set sel_poly true
 set sel_arc true
 set sel_spline true
 set sel_path true
+set sheetsize letter
 
 set m [menu .xcircuit.menubar.filebutton.filemenu -tearoff 0]
 $m add command -label "Read XCircuit File" -command {xcircuit::promptloadfile}
 $m add command -label "Import XCircuit PS" -command {xcircuit::promptimportfile}
 $m add command -label "Import background PS" -command {xcircuit::promptimportbackground}
 $m add command -label "Execute Script" -command {xcircuit::promptexecscript}
+$m add command -label "Tcl Console" -command {xcircuit::raiseconsole}
 $m add command -label "Write XCircuit PS" -command {xcircuit::promptsavepage}
 $m add separator
 $m add command -label "Add To Library" -command {xcircuit::promptaddlibrary}
 $m add command -label "Load New Library" -command {xcircuit::promptloadlibrary}
-$m add command -label "Save Library" -command {xcircuit::promptsavelib}
+$m add command -label "Save Library" -command {if {[catch xcircuit::library]} \
+	{xcircuit::promptsavelib [xcircuit::library "User Library"]} \
+	{xcircuit::promptsavelib [xcircuit::library]}}
 $m add separator
 $m add command -label "Clear Page" -command {xcircuit::page reset}
 $m add separator
@@ -1067,9 +1297,9 @@ $m add cascade -label "Encoding" -menu $m.encodingmenu
 $m add cascade -label "Insert" -menu $m.insertmenu
 $m add cascade -label "Justification" -menu $m.justifymenu
 $m add command -label "Parameterize" \
-	-command {xcircuit::label parameter make substring}
+	-command {xcircuit::parameter make substring}
 $m add command -label "Unparameterize" \
-	-command {xcircuit::label parameter forget substring}
+	-command {xcircuit::parameter forget substring}
 $m add separator
 $m add command -label "Make Label" -command {xcircuit::label make}
 
@@ -1204,7 +1434,7 @@ $m3 add separator
 $m3 add check -label "Closed" -variable polyclosed -onvalue closed \
 	-offvalue unclosed -command {xcircuit::border  $polyclosed}
 $m3 add check -label "Bounding Box" -variable bboxtype -onvalue true \
-	-offvalue false -command {xcircuit::borderr bbox $bboxtype}
+	-offvalue false -command {xcircuit::border bbox $bboxtype}
 
 # stipple images
 image create bitmap img_stip0 -foreground white -background black -file \
@@ -1353,57 +1583,8 @@ $m2 add command -label "Info label" -command {xcircuit::label type info}
 # Create the toolbar and associated tools
 #-----------------------------------------
 
-set tools [list b a s t cp e d2 cw ccw fx fy \
-	pu2 po2 mk pz uj co bd fi pm li yp pl z4 z5 i]
-set tooltips [list "draw box" "draw arc" "draw spline" "enter text" \
-	"copy element" "edit element" "delete element" \
-	"rotate 15 degrees clockwise" "rotate 15 degrees counterclockwise" \
-	"flip horizontal" "flip vertical" "push (edit object)" \
-	"pop (return from object edit)" "make an object from selection" \
-	"join elements into polygon or path" "separate path into elements" \
-	"set color" "set border and line properties" "set fill properties" \
-	"parameterize properties" "go to next library" "go to library directory" \
-	"go to page directory" "zoom in" "zoom out" "pop up help window"]	
-set toolactions [list {xcircuit::polygon make box} {xcircuit::arc make} \
-	{xcircuit::spline make} {xcircuit::label make} {xcircuit::copy selected} \
-	{xcircuit::edit selected} {xcircuit::delete selected} {xcircuit::rotate 15} \
-	{xcircuit::rotate -15} {xcircuit::flip horizontal} \
-	{xcircuit::flip vertical} {xcircuit::push selected} {xcircuit::pop} \
-	{xcircuit::promptmakeobject} {xcircuit::path join selected} \
-	{xcircuit::path unjoin selected} \
-	{tk_popup .colormenu [expr {[winfo rootx .xcircuit.mainframe.toolbar.bco] \
-		- [winfo width .colormenu]}] \
-		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bco] \
-		- [winfo height .colormenu] / 2}] } \
-	{tk_popup .bordermenu [expr {[winfo rootx .xcircuit.mainframe.toolbar.bbd] \
-		- [winfo width .bordermenu]}] \
-		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bbd] \
-		- [winfo height .bordermenu] / 2}] } \
-	{tk_popup .fillmenu [expr {[winfo rootx .xcircuit.mainframe.toolbar.bfi] \
-		- [winfo width .fillmenu]}] \
-		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bfi] \
-		- [winfo height .fillmenu] / 2}] } \
-	{tk_popup .parammenu [expr {[winfo rootx .xcircuit.mainframe.toolbar.bpm] \
-		- [winfo width .parammenu]}] \
-		[expr {[winfo rooty .xcircuit.mainframe.toolbar.bpm] \
-		- [winfo height .parammenu] / 2}] } \
-	{xcircuit::library next} \
-	{xcircuit::library directory} {xcircuit::page directory} \
-	{xcircuit::zoom 1.5; refresh} {xcircuit::zoom [expr {1 / 1.5}]; refresh} \
-	{wm deiconify .help} ]
-
-for {set i 0} {$i < [llength $tools]} {incr i 1} {
-   set bname [lindex $tools $i]
-   set btip [lindex $tooltips $i]
-   image create xpm img_${bname} -file ${XCIRCUIT_LIB_DIR}/pixmaps/${bname}.xpm
-   button .xcircuit.mainframe.toolbar.b${bname} -image img_${bname} \
-	 -command [lindex $toolactions $i]
-   pack .xcircuit.mainframe.toolbar.b${bname} -side top
-   bind .xcircuit.mainframe.toolbar.b${bname} <Enter> \
-	[subst {.xcircuit.infobar.message2 configure -text "$btip"}]
-   bind .xcircuit.mainframe.toolbar.b${bname} <Leave> \
-	[subst {.xcircuit.infobar.message2 configure -text ""}]
-}
+xcircuit::createtoolbar
+xcircuit::arrangetoolbar
 
 #-----------------------------------------------------------------
 # Final setup stuff before exiting back to interpreter
@@ -1429,6 +1610,11 @@ xcircuit::newlibrarybutton "User Library"
 bind .xcircuit.mainframe.mainarea.drawing <Enter> {focus %W}
 
 tkwait visibility .xcircuit.mainframe.mainarea.drawing
+focus .xcircuit.mainframe.mainarea.drawing
+
+# This pops down the interpreter window, if the "console.tcl" script was run.
+
+catch xcircuit::consoledown
 
 # End of GUI configuration file.  Xcircuit continues to load the xcircuit
 # startup configuration files.
