@@ -11,11 +11,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -42,6 +37,9 @@
 /* Externally declared global variables					*/
 /*----------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER
+extern Tcl_Interp *xcinterp;
+#endif
 extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern Widget     menuwidgets[];
@@ -53,7 +51,7 @@ extern char _STR[150];
 /* types which are able to accept the given parameterization.		*/
 /*----------------------------------------------------------------------*/
 
-static u_char param_select[] = {
+u_char param_select[] = {
    POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION */
    LABEL,					/* P_SUBSTRING */
    POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION_X */
@@ -143,7 +141,30 @@ void indicateparams(genericptr thiselem)
 #ifdef TCL_WRAPPER
 void setparammarks(genericptr thiselem)
 {
-   /* to be done! */
+   /* Set GUI variables associated with the "parameter" menu.	*/
+
+   int i, j, paramno;
+   oparamptr ops;
+
+   /* These match the order of parameter definitions in xcircuit.h */
+   const char *paramvars[] = {"positionparam", "substringparam", "xparam",
+	"yparam", "styleparam", "justparam", "startparam", "endparam",
+	"radiusparam", "minorparam", "rotationparam", "scaleparam",
+	"linewidthparam", "colorparam"};
+
+   /* Reset all variables to false */
+   for (i = 0; i < 14; i++)
+      Tcl_SetVar(xcinterp, (char *)paramvars[i], "false", TCL_NAMESPACE_ONLY);
+
+   /* For each parameter declared, set the corresponding Tcl variable */
+   if (thiselem != NULL) {
+      for (i = 0; i < thiselem->num_params; i++) {
+         j = thiselem->passed[i].paramno;
+         ops = topobject->params[j];
+         Tcl_SetVar(xcinterp, (char *)paramvars[ops->which], "true",
+		TCL_NAMESPACE_ONLY);
+      }
+   }
 }
 #else
 
@@ -689,7 +710,8 @@ void resolveparams(objinstptr thisinst)
 	    break;
 
       if ((spec == NULL) || (spec->virtual == FALSE)) {
-         /* printf("Came from library or top page:  changing default value\n"); */
+         /* Fprintf(stdout, "Came from library or top page:  "
+			"changing default value\n"); */
          replaceparams(thisinst);
          return;
       }
@@ -788,7 +810,7 @@ void copyparams(objinstptr newinst, objinstptr callinst)
 			   callinst->params[i]->parameter.ivalue;
 	       break;
 	    default:
-	       printf("Error:  bad parameter\n");
+	       Fprintf(stderr, "Error:  bad parameter\n");
 	       break;
 	 }
       }
@@ -824,7 +846,7 @@ void makenumericalp(genericptr *gelem, u_int mode)
 		+ (*gelem)->num_params; epp++) {
 	 ops = *(topobject->params + epp->paramno);
 	 if (ops->which == (u_char)mode) {
-	    fprintf(stderr, "Cannot duplicate a parameter!\n");
+	    Fprintf(stderr, "Cannot duplicate a parameter!\n");
 	    return;
 	 }
       }
@@ -987,7 +1009,7 @@ void makenumericalp(genericptr *gelem, u_int mode)
 	 }
 	 break;
    }
-
+   incr_changes(topobject);
 }
 
 /*--------------------------------------------------------------*/
@@ -999,6 +1021,7 @@ void unmakenumericalp(genericptr *gelem, u_int mode)
    /* Parameterized strings are handled by makeparam() */
    if (((*gelem)->type == LABEL) && (mode == P_SUBSTRING)) {
       unparameterize(mode);
+      setparammarks(*gelem);
       return;
    }
 
@@ -1014,7 +1037,7 @@ void insertparam()
    labelptr tlab;
    oparamptr ops;
    short i, nparms = 0;
-   int selparm = -1;
+   int result, selparm = -1;
 
    /* Don't allow nested parameters */
 
@@ -1062,9 +1085,29 @@ void insertparam()
       Wprintf(sstart);
       free(sstart);
 
-      /* This is a bit quirky;  but works for now.  Later, would prefer */
-      /* a GUI-oriented selection mechanism.                            */
+#ifdef TCL_WRAPPER
+
+      /* The Tcl version uses a GUI selection mechanism. */
+
+      result = Tcl_Eval(xcinterp, "xcircuit::promptselectparam");
+      if (result != TCL_OK) {
+	 Tcl_SetResult(xcinterp, "Error in executing promptselectparam", NULL);
+         return;
+      }
+      result = Tcl_GetIntFromObj(xcinterp, Tcl_GetObjResult(xcinterp), &selparm);
+      if (result != TCL_OK) {
+	 Tcl_SetResult(xcinterp, "Bad parameter number.", NULL);
+	 selparm = -1;
+      }
+      /* Relate index in list (string parameters only) to index	*/
+      /* over all parameters.					*/
+
+      if (selparm >= 0) selparm = parammap[selparm];
+#else
+      /* The non-Tcl version is a bit quirky, but it works. */
+
       selparm = parammap[getkeynum()];
+#endif
       free(parammap);
    }
 
@@ -1146,11 +1189,13 @@ void makeparam(labelptr thislabel)
 
    ops = *(topobject->params + topobject->num_params - 1);
    ops->type = XC_STRING;
+   ops->which = P_SUBSTRING;
    ops->parameter.string = begpart->nextpart;
    begpart->nextpart = endpart->nextpart;
    endpart->nextpart = NULL;
 
    textend = 0;
+   incr_changes(topobject);
 }
 
 /*------------------------------------------------------*/
@@ -1402,6 +1447,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
       }
    }
    freelabel(subs);
+   incr_changes(topobject);
 }
 
 /*------------------------------------------------------*/
@@ -1462,6 +1508,7 @@ void unparameterize(int mode)
 	 else
 	    unmakenumericalp(topobject->plist + (*fselect), mode);
       }
+      setparammarks(NULL);
    }
 }
 
@@ -1497,6 +1544,7 @@ void parameterize(int mode)
 	 makenumericalp(topobject->plist + (*fselect), mode);
    }
    objectdeselect();
+   setparammarks(NULL);
 }
 
 /*----------------------------------------------------------------------*/
@@ -1535,7 +1583,7 @@ int checklibtop()
    int i;
    pushlistptr thispush;
 
-   for (thispush = xobjs.stack; thispush != NULL; thispush = thispush->next)
+   for (thispush = areastruct.stack; thispush != NULL; thispush = thispush->next)
       if ((i = is_library(thispush->thisinst->thisobject)) >= 0)
 	 return i;
 
