@@ -11,6 +11,7 @@ extern Tk_EventProc *xctk_drag(ClientData, XEvent *);
 extern Tk_EventProc *xctk_draglscroll(ClientData, XEvent *);
 extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
 extern int XcTagCallback(Tcl_Interp *, int, Tcl_Obj *CONST objv[]);
+extern objectptr NameToObject(char *, objinstptr *, Boolean);
 
 #endif
 
@@ -201,10 +202,11 @@ extern char *varscan(objectptr, char *, short *, genericptr, u_char);
 extern char *varfscan(objectptr, char *, float *, genericptr, u_char);
 extern objinstptr addtoinstlist(int, objectptr, Boolean);
 extern Boolean objectread(FILE *, objectptr, short, short, short, char *, int);
-extern void setfile(xcWidget, xcWidget, caddr_t);
 #ifdef TCL_WRAPPER
+void setfile(char *, int);
 extern xcTimeOutProc savetemp(caddr_t);
 #else
+extern void setfile(xcWidget, xcWidget, caddr_t);
 extern xcTimeOutProc savetemp(caddr_t, xcIntervalId *);
 #endif
 extern void savelibpopup(xcWidget, pointertype, caddr_t);
@@ -347,6 +349,7 @@ extern short findhelvetica();
 extern void catreturn();
 extern int pageposition(short, XButtonEvent *, int);
 extern short pagelinks();
+extern short *pagetotals(int, short);
 extern int is_page(objectptr);
 extern int is_library(objectptr);
 extern void pagecatbutton(XButtonEvent *);
@@ -404,11 +407,14 @@ extern void setpagelabel(xcWidget, char *);
 extern void makenewfontbutton();
 extern void makenewencodingbutton(char *, char);
 extern void togglefontmark(int);
+#ifndef TCL_WRAPPER
 extern void toggleexcl(xcWidget, menuptr, int);
+#endif
 extern void toggle(xcWidget, Boolean *, caddr_t);
 extern void setcolorscheme(Boolean);
 extern void inversecolor(xcWidget, Boolean *, caddr_t);
 extern void togglegrid(u_short);
+extern float parseunits(char *);
 extern void setgridtype(char *);
 extern void getgridtype(xcWidget, pointertype, caddr_t);
 extern void newlibrary(xcWidget, caddr_t, caddr_t);
@@ -599,7 +605,8 @@ extern void pinconvert(labelptr, pointertype);
 extern void dopintype(xcWidget, pointertype, caddr_t);
 extern void setsymschem();
 extern int findpageobj(objectptr);
-extern int findsubschems(int, objectptr, int);
+extern void collectsubschems(int);
+extern int findsubschems(int, objectptr, int, short *);
 extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
@@ -608,7 +615,7 @@ extern void swapschem(xcWidget, pointertype, caddr_t);
 extern void dobeforeswap(xcWidget, caddr_t, caddr_t);
 extern void schemdisassoc();
 extern void startschemassoc(xcWidget, pointertype, caddr_t);
-extern void schemassoc(objectptr, objectptr);
+extern Boolean schemassoc(objectptr, objectptr);
 
 /* from selection.c: */
 
@@ -616,7 +623,7 @@ extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
 extern void geneasydraw(short, short, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
-extern short *selectobject(short, objectptr, objinstptr);
+extern short *genobjectselect(short, objectptr, objinstptr);
 extern short *allocselect();
 extern void setoptionmenu();
 extern int test_insideness(int, int, XPoint *);
@@ -697,6 +704,7 @@ extern void writescalevalues(char *, char *, char *);
 extern int Tcl_GetHandleFromObj(Tcl_Interp *, Tcl_Obj *, void **);
 #else
 extern void updatetext(xcWidget, xcWidgetList, caddr_t);
+extern void delwin(xcWidget, popupstruct *, XClientMessageEvent *);
 #endif
 
 extern void makecursors();
