@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------*/
 /* python.c --- Embedded python interpreter for xcircuit	*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University    */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University    */
 /*--------------------------------------------------------------*/
 /* NOTE: These routines work only if python-2.0 is installed.   */
 /*	Requires library file libpython2.0.a.  Hopefully	*/
@@ -11,17 +11,12 @@
 /*	Modeled after demo.c in Python2.0 source Demo/embed	*/
 /*--------------------------------------------------------------*/
 
-#ifdef HAVE_PYTHON
+#if defined(HAVE_PYTHON) && !defined(TCL_WRAPPER)
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>	/* for usleep() */
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
    
 #include <Python.h>
 
@@ -32,33 +27,7 @@
       
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-/* "unknown" tool */
-static char *q_xpm[] = {
-"20 20 4 1",
-".      c #FFFFFFFFFFFF",
-"X      c #C2C2CECEFFFF",
-"o      c #79799090FFFF",
-"O      c #00000000FFFF",
-"....................",
-".......XXXX.........",
-".....XoOOOOoX.......",
-".....oOOoXOOOX......",
-"....oOOX..XOOo......",
-"...XOOX....XOO......",
-"...XOO.....XOO......",
-"....XX.....oOo......",
-"..........XOO.......",
-".........XOO........",
-"........XOOX........",
-".......XOO..........",
-".......OOX..........",
-".......oOOXo........",
-".......XOOO.........",
-"........XX..........",
-".......XOOX.........",
-".......oOOo.........",
-".......XOOX.........",
-"...................."};
+#include "lib/pixmaps/q.xpm"
 #endif
 
 PyObject *xcmod;
@@ -69,6 +38,7 @@ PyObject *xcmod;
       
 #include "xcircuit.h"
 #include "menudep.h"
+#include "colordefs.h"
 
 /*----------------------------------------------------------------------*/
 /* Function prototype declarations                                      */
@@ -91,6 +61,7 @@ static PyObject *xc_popupprompt(PyObject *, PyObject *);
 static PyObject *xc_filepopup(PyObject *, PyObject *);
 static PyObject *xc_newbutton(PyObject *, PyObject *);
 static PyObject *xc_page(PyObject *, PyObject *);
+static PyObject *xc_load(PyObject *, PyObject *);
 static PyObject *xc_pan(PyObject *, PyObject *);
 static PyObject *xc_zoom(PyObject *, PyObject *);
 static PyObject *xc_getcursor();
@@ -115,8 +86,8 @@ extern Globaldata xobjs;
 extern short beeper; 
 extern Widget menuwidgets[];
 extern Display *dpy;
-extern Window win;
 extern keybinding *keylist;
+extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
 extern ApplicationData appdata;
@@ -134,6 +105,21 @@ short flags = -1;
 #define FONTOVERRIDE    8
 #define KEYOVERRIDE	16
 
+/*----------------------------------------------------------------*/
+/* Similar to "PyRun_SimpleString()" but returns a PyObject value */
+/* rather than a status word.					  */
+/*----------------------------------------------------------------*/
+
+PyObject *PyRun_ObjString(char *string)
+{
+   PyObject *m, *d, *v;
+   m = PyImport_AddModule("__main__");
+   if (m == NULL) return NULL;
+   d = PyModule_GetDict(m);
+   v = PyRun_String(string, Py_file_input, d, d);
+   return v;
+}
+
 /*--------------------------------------------------------------*/
 /* Define all of the xcircuit functions available to Python	*/
 /* Currently this does the bare minimum of having Python do the	*/
@@ -224,32 +210,32 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
    
    switch(type = string_to_type(etype)) {
       case ARC:
-         NEW_ARC(newarc, objectdata);
+         NEW_ARC(newarc, topobject);
 	 arcdefaults(*newarc, 0, 0);
 	 newgen = (genericptr *)newarc;
 	 break;
       case SPLINE:
-         NEW_SPLINE(newspline, objectdata);
+         NEW_SPLINE(newspline, topobject);
 	 splinedefaults(*newspline, 0, 0);
 	 newgen = (genericptr *)newspline;
 	 break;
       case POLYGON:
-         NEW_POLY(newpoly, objectdata);
+         NEW_POLY(newpoly, topobject);
 	 polydefaults(*newpoly, 4, 0, 0);
 	 newgen = (genericptr *)newpoly;
 	 break;
       case LABEL:
-         NEW_LABEL(newlabel, objectdata);
+         NEW_LABEL(newlabel, topobject);
 	 labeldefaults(*newlabel, 0, 0, 0);
 	 newgen = (genericptr *)newlabel;
 	 break;
       case PATH:
-         NEW_PATH(newpath, objectdata);
+         NEW_PATH(newpath, topobject);
 	 pathdefaults(*newpath, 0, 0);
 	 newgen = (genericptr *)newpath;
 	 break;
       case OBJECT:
-         NEW_OBJINST(newobjinst, objectdata);
+         NEW_OBJINST(newobjinst, topobject);
 	 objectdefaults(*newobjinst, NULL, 0, 0);
 	 newgen = (genericptr *)newobjinst;
 	 break;
@@ -258,8 +244,12 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
 		"newelement() 2nd argument must be a valid element type");
          return NULL;
    } 
-   objectdata->parts++;
-   incr_changes(objectdata);
+   topobject->parts++;
+   incr_changes(topobject);
+#ifdef SCHEMA
+   topobject->valid = False;
+#endif
+
    return PyInt_FromLong((long)(*newgen));
 }
 
@@ -621,14 +611,15 @@ genericptr *CheckHandle(PyObject *ehandle)
    Library *thislib;
 
    eaddr = (int)PyInt_AsLong(ehandle);
-   for (gelem = objectdata->plist; gelem < objectdata->plist +
-	objectdata->parts; gelem++)
+   for (gelem = topobject->plist; gelem < topobject->plist +
+	topobject->parts; gelem++)
       if ((int)(*gelem) == eaddr) goto exists;
 
-   /* Okay, it isn't in objectdata.  Try all other pages. */
+   /* Okay, it isn't in topobject.  Try all other pages. */
 
    for (i = 0; i < xobjs.pages; i++) {
-      if (!(thisobj = xobjs.pagelist[i]->pageobj)) continue;
+      if (xobjs.pagelist[i]->pageinst == NULL) continue;
+      thisobj = xobjs.pagelist[i]->pageinst->thisobject;
       for (gelem = thisobj->plist; gelem < thisobj->plist + thisobj->parts; gelem++)
          if ((int)(*gelem) == eaddr) goto exists;
    }
@@ -707,7 +698,7 @@ static PyObject *xc_getpage(PyObject *self, PyObject *args)
 	PyString_FromString(thispage->filename));
 
    PyDict_SetItem(rdict, PyString_FromString("page label"),
-	PyString_FromString(thispage->pageobj->name));
+	PyString_FromString(thispage->pageinst->thisobject->name));
 
    PyDict_SetItem(rdict, PyString_FromString("output scale"),
 	PyFloat_FromDouble((double)thispage->outscale));
@@ -761,12 +752,13 @@ static PyObject *xc_getlibrary(PyObject *self, PyObject *args)
    }
    else {
       for (lpage = 0; lpage < xobjs.numlibs; lpage++) {
-	 if (!strcmp(lname, xobjs.libtop[lpage + LIBRARY]->name)) {
+	 if (!strcmp(lname, xobjs.libtop[lpage + LIBRARY]->thisobject->name)) {
 	    curlib = xobjs.userlibs[lpage - 1].library;
 	    break;
 	 }
-	 else if (!strncmp(xobjs.libtop[lpage + LIBRARY]->name, "Library: ", 9)) {
-	    if (!strcmp(lname, xobjs.libtop[lpage + LIBRARY]->name + 9)) {
+	 else if (!strncmp(xobjs.libtop[lpage + LIBRARY]->thisobject->name,
+			"Library: ", 9)) {
+	    if (!strcmp(lname, xobjs.libtop[lpage + LIBRARY]->thisobject->name + 9)) {
 	       curlib = xobjs.userlibs[lpage - 1].library;
 	       break;
 	    }
@@ -786,7 +778,7 @@ static PyObject *xc_getlibrary(PyObject *self, PyObject *args)
    rdict = PyDict_New();
 
    PyDict_SetItem(rdict, PyString_FromString("name"),
-	PyString_FromString(xobjs.libtop[lpage + LIBRARY]->name));
+	PyString_FromString(xobjs.libtop[lpage + LIBRARY]->thisobject->name));
 
    if (curobjs > 0) {
       dlist = PyList_New(curobjs);
@@ -826,9 +818,9 @@ static PyObject *xc_getobject(PyObject *self, PyObject *args)
 
    if (thisobj == NULL) { 	/* try the page objects */
       for (k = 0; k < xobjs.pages; k++) {
-         if (!(thisobj = xobjs.pagelist[k]->pageobj)) continue;
-	 if (!strcmp(oname, thisobj->name))
-	    break;
+	 if (xobjs.pagelist[k]->pageinst != NULL)
+	    if (!strcmp(oname, xobjs.pagelist[k]->pageinst->thisobject->name))
+	       break;
       }
       if (k == xobjs.pages) return NULL;	/* not found */
    }
@@ -840,15 +832,15 @@ static PyObject *xc_getobject(PyObject *self, PyObject *args)
    PyDict_SetItem(rdict, PyString_FromString("name"),
 	PyString_FromString(thisobj->name));
    PyDict_SetItem(rdict, PyString_FromString("width"), 
-	PyInt_FromLong((long)(thisobj->width)));
+	PyInt_FromLong((long)(thisobj->bbox.width)));
    PyDict_SetItem(rdict, PyString_FromString("height"), 
-	PyInt_FromLong((long)(thisobj->height)));
+	PyInt_FromLong((long)(thisobj->bbox.height)));
    PyDict_SetItem(rdict, PyString_FromString("viewscale"), 
 	PyFloat_FromDouble((double)(thisobj->viewscale)));
 
    tpos = PyTuple_New(2);
-   PyTuple_SetItem(tpos, 0, PyInt_FromLong((long)thisobj->lowerleft.x));
-   PyTuple_SetItem(tpos, 1, PyInt_FromLong((long)thisobj->lowerleft.y));
+   PyTuple_SetItem(tpos, 0, PyInt_FromLong((long)thisobj->bbox.lowerleft.x));
+   PyTuple_SetItem(tpos, 1, PyInt_FromLong((long)thisobj->bbox.lowerleft.y));
    PyDict_SetItem(rdict, PyString_FromString("boundingbox"), tpos);
 
    tpos = PyTuple_New(2);
@@ -885,7 +877,7 @@ static PyObject *xc_getobject(PyObject *self, PyObject *args)
 
 static PyObject *xc_reset()
 {
-   resetbutton(NULL, NULL, NULL);
+   resetbutton(NULL, (pointertype)0, NULL);
    return PyInt_FromLong(0L);
 }
 
@@ -902,22 +894,46 @@ static PyObject *xc_netlist()
 {
    PyObject *rdict = NULL;
 
-   if (cleartraversed(objectdata, 0) == -1)
-      PyErr_SetString(PyExc_TypeError, "Error:  Check circuit for infinite recursion.");
-   else {
-      createnets(False, "spice");
-      subindex = 1;
-   
-      rdict = PyDict_New();
-      PyDict_SetItem(rdict, PyString_FromString("globals"), pyglobals(objectdata));
-      PyDict_SetItem(rdict, PyString_FromString("circuit"), pytoplevel(objectdata));
-
-      destroynets();
+   if (checkvalid(topobject) == -1) {
+      if (cleartraversed(topobject, 0) == -1)
+         PyErr_SetString(PyExc_TypeError, "Error:  Check circuit for infinite recursion.");
+      else {
+         destroynets();
+         createnets();
+      }
    }
+
+   subindex = 1;
+   rdict = PyDict_New();
+   PyDict_SetItem(rdict, PyString_FromString("globals"), pyglobals(topobject));
+   PyDict_SetItem(rdict, PyString_FromString("circuit"), pytoplevel(topobject));
    return rdict;
 }
 #endif
 
+/*--------------------------------------------------------------*/
+/* Load the specified file					*/
+/*--------------------------------------------------------------*/
+
+static PyObject *xc_load(PyObject *self, PyObject *args)
+{
+   char *filename;
+   int pageno = 0, savepage;
+
+   if (!PyArg_ParseTuple(args, "s|i:load", &filename, &pageno))
+      return PyInt_FromLong((long)areastruct.page);
+
+   if (--pageno < 0) pageno = areastruct.page;
+   savepage = areastruct.page;
+
+   strcpy(_STR2, filename);
+   if (savepage != pageno) newpage(pageno);
+   startloadfile();
+   if (savepage != pageno) newpage(savepage);
+
+   return PyString_FromString(filename);
+}
+
 /*--------------------------------------------------------------*/
 /* Go to the specified page					*/
 /*--------------------------------------------------------------*/
@@ -961,7 +977,7 @@ static PyObject *xc_zoom(PyObject *self, PyObject *args)
    }
 
    areastruct.zoomfactor = save;
-   return PyFloat_FromDouble((double)objectdata->viewscale);
+   return PyFloat_FromDouble((double)topobject->viewscale);
 }
 
 /*--------------------------------------------------------------*/
@@ -1408,7 +1424,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	 break;
    }
 
-   calcbbox(objectdata);
+   calcbbox(areastruct.topinstance);
    return PyInt_FromLong((long)(*gelem));
 }
 /*--------------------------------------------------------------*/
@@ -1508,7 +1524,7 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
 #ifdef SCHEMA
    else if (!strcmp(sarg1, "xschema") || !strcmp(sarg1, "schema")) {
       areastruct.schemon = a;
-      doxschema(OptionsEnableXSchemaButton, NULL, NULL);
+      doxschema(OptionsEnableXSchemaButton, 1, NULL);
    }
 #endif
 #ifdef HAVE_XPM
@@ -1718,7 +1734,7 @@ static PyObject *xc_refresh(PyObject *self, PyObject *args)
       
    refresh(NULL, NULL, NULL);
 
-   while(XCheckWindowEvent(dpy, win, ~NoEventMask, &event))
+   while(XCheckWindowEvent(dpy, areastruct.areawin, ~NoEventMask, &event))
          XtDispatchEvent(&event);
 
    return PyInt_FromLong(0L);
@@ -1851,17 +1867,21 @@ static PyObject *xc_newtool(PyObject *self, PyObject *args)
 Widget *pytoolbuttons(int *num_tools)
 {
    Widget *ptools;
-   PyObject *rval, *pdict, *pmod;
+   PyObject *rval;
    int i;
 
    if (xcmod == NULL) {
       *num_tools = 0;
       return NULL;
    }
-   pdict = PyModule_GetDict(xcmod);
-   rval = PyRun_String("len(tooldict)", Py_eval_input, pdict, pdict); /* ? */
+   rval = PyRun_ObjString("len(tooldict)");
+   if (!rval) {
+      PyErr_Clear();
+      return NULL;
+   }
    *num_tools = (int)PyInt_AsLong(rval);
-   fprintf(stderr, "num_tools = %d\n", *num_tools);
+   Py_DECREF(rval);
+   /* fprintf(stderr, "num_tools = %d\n", *num_tools); */
 
    if (*num_tools <= 0) return NULL;
 
@@ -1869,8 +1889,13 @@ Widget *pytoolbuttons(int *num_tools)
 
    for (i = 0; i < *num_tools; i++) {
       sprintf(_STR2, "tooldict[%d]", i);
-      rval = PyRun_String(_STR2, Py_eval_input, pdict, pdict);
+      rval = PyRun_ObjString(_STR2);
+      if (!rval) {
+         PyErr_Clear();
+	 return NULL;
+      }
       ptools[i] = (Widget)((int)PyInt_AsLong(rval));
+      Py_DECREF(rval);
    }
    return ptools;
 }
@@ -1883,11 +1908,20 @@ Widget *pytoolbuttons(int *num_tools)
 
 int python_key_command(int keystate)
 {
+   PyObject *rval;
    int status;
+   sprintf(_STR2, "keydict[%d]\n", keystate);
+   rval = PyRun_ObjString(_STR2);
+   if (!rval) {
+      PyErr_Clear();
+      return -1;
+   }
+   Py_DECREF(rval);
+
    sprintf(_STR2, "keydict[%d]()\n", keystate);
-   status = PyRun_SimpleString(_STR2);
+   PyRun_SimpleString(_STR2);
    refresh(NULL, NULL, NULL);
-   return status;
+   return 0;
 }
 
 /*--------------------------------------------------------------*/
@@ -1920,6 +1954,7 @@ static PyMethodDef xc_methods[] = {
    {"newbutton",	xc_newbutton,	1},
    {"reset",		xc_reset,	1},
    {"page",		xc_page,	1},
+   {"load",		xc_load,	1},
 #ifdef HAVE_XPM
    {"newtool",		xc_newtool,	1},
 #endif
@@ -1979,17 +2014,22 @@ short execcommand(short pflags, char *cmdptr)
 void defaultscript()
 {
    FILE *fd;
+   char *tmp_s = getenv((const char *)"XCIRCUIT_LIB_DIR");
 
    flags = LIBOVERRIDE | LIBLOADED | FONTOVERRIDE;
 
-   sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
+   if (!tmp_s) tmp_s = BUILTINS_DIR; 
+   sprintf(_STR2, "%s/%s", tmp_s, STARTUP_FILE);
 
-   if ((fd = fopen(_STR2, "r")) != NULL)
-      PyRun_SimpleFile(fd, _STR2);
-   else {
-      sprintf(_STR, "Failed to open startup script \"%s\"\n", STARTUP_FILE);
-      Wprintf(_STR);
+   if ((fd = fopen(_STR2, "r")) == NULL) {
+      sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
+      if ((fd = fopen(_STR2, "r")) == NULL) {
+         sprintf(_STR, "Failed to open startup script \"%s\"\n", STARTUP_FILE);
+         Wprintf(_STR);
+	 return;
+      }
    }
+   PyRun_SimpleFile(fd, _STR2);
 }
 
 /*----------------------------------------------------------------------*/
@@ -2002,7 +2042,7 @@ void execscript()
 
    flags = 0;
 
-   tilde_expand(_STR2);
+   xc_tilde_expand(_STR2);
    if ((fd = fopen(_STR2, "r")) != NULL) {
       PyRun_SimpleFile(fd, _STR2);
       refresh(NULL, NULL, NULL);
@@ -2031,10 +2071,12 @@ void loadrcfile()
       
    /* try first in current directory, then look in user's home directory */
       
-   tilde_expand(_STR2);
+   xc_tilde_expand(_STR2);
    if ((fd = fopen(_STR2, "r")) == NULL) {
-      sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
-      fd = fopen(_STR2, "r");
+      if (userdir != NULL) {
+         sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
+         fd = fopen(_STR2, "r");
+      }
    }  
    if (fd != NULL)
       PyRun_SimpleFile(fd, _STR2);
@@ -2069,20 +2111,29 @@ void loadrcfile()
       addnewcolorentry(xc_alloccolor("Gray90"));
       addnewcolorentry(xc_alloccolor("Red"));
       addnewcolorentry(xc_alloccolor("Blue"));
-      addnewcolorentry(xc_alloccolor("Green"));
+      addnewcolorentry(xc_alloccolor("Green2"));
       addnewcolorentry(xc_alloccolor("Yellow"));
       addnewcolorentry(xc_alloccolor("Purple"));
-      addnewcolorentry(xc_alloccolor("Cyan"));
-      addnewcolorentry(xc_alloccolor("Magenta"));
-      addnewcolorentry(xc_alloccolor("Orange"));
+      addnewcolorentry(xc_alloccolor("SteelBlue2"));
+      addnewcolorentry(xc_alloccolor("Red3"));
       addnewcolorentry(xc_alloccolor("Tan"));
       addnewcolorentry(xc_alloccolor("Brown"));
    }  
 
+#ifdef SCHEMA
+   /* These colors must be enabled whether or not colors are overridden, */
+   /* because they are needed by the schematic capture system.		 */
+
+   addnewcolorentry(xc_getlayoutcolor(LOCALPINCOLOR));
+   addnewcolorentry(xc_getlayoutcolor(GLOBALPINCOLOR));
+   addnewcolorentry(xc_getlayoutcolor(INFOLABELCOLOR));
+#endif
+   addnewcolorentry(xc_getlayoutcolor(BBOXCOLOR));
+
    if (!(flags & KEYOVERRIDE))
       default_keybindings();
 }
 
 #endif
-/* #endif HAVE_PYTHON */
+/* #endif HAVE_PYTHON and !TCL_WRAPPER */
 /*--------------------------------------------------------------*/
