@@ -1,10 +1,10 @@
 /*-------------------------------------------------------------------------*/
 /* rcfile.c --- Load initialization variables from the .xcircuitrc file	   */
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /* This file is made obsolete by the Python interpreter, if compiled in */
-#ifndef HAVE_PYTHON
+#if !defined(HAVE_PYTHON) && !defined(TCL_WRAPPER)
    
 /*-------------------------------------------------------------------------*/
 /*      written by Tim Edwards, 9/9/96                                     */
@@ -13,15 +13,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes                                                       */
@@ -29,6 +29,7 @@
 
 #include "xcircuit.h"
 #include "menudep.h"
+#include "colordefs.h"
 
 /*----------------------------------------------------------------------*/
 /* Function prototype declarations                                      */
@@ -45,9 +46,9 @@ extern short fontcount;
 extern Clientdata areastruct;
 extern Globaldata xobjs;
 extern short beeper;
+extern int *appcolors;
 extern Widget menuwidgets[];
 extern Display *dpy;
-extern Window win;
 
 /*----------------------------------------------------------------------*/
 
@@ -104,7 +105,7 @@ short execcommand(short pflags, char *cmdptr)
 #ifdef SCHEMA
       else if (!strcmp(_STR, "xschema") || !strcmp(_STR, "schema")) {
 	 areastruct.schemon = a ? True : False;
-	 doxschema(OptionsEnableXSchemaButton, NULL, NULL);
+	 doxschema(OptionsEnableXSchemaButton, 1, NULL);
       }
 #endif
 #ifdef HAVE_XPM
@@ -141,6 +142,20 @@ short execcommand(short pflags, char *cmdptr)
       }
    }
 
+   /* load extra files */
+
+   else if (!strcmp(cmdptr, "load")) {
+      int pageno = areastruct.page;
+      int savepage = areastruct.page;
+      if (sscanf(argptr, "%249s %d", _STR, &pageno) >= 1) {
+	 if (pageno != savepage) newpage(pageno);
+	 startloadfile();
+	 if (pageno != savepage) newpage(savepage);
+      }
+      else
+	 Wprintf("Error in load statement.");
+   }
+
    /* load builtin libraries */
 
    else if (!strcmp(cmdptr, "library")) {
@@ -255,7 +270,7 @@ short execcommand(short pflags, char *cmdptr)
 
       refresh(NULL, NULL, NULL);
 
-      while(XCheckWindowEvent(dpy, win, ~NoEventMask, &event))
+      while(XCheckWindowEvent(dpy, areastruct.areawin, ~NoEventMask, &event))
          XtDispatchEvent(&event);
    }
 
@@ -394,14 +409,20 @@ short readcommand(short mode, FILE *fd)
 void defaultscript()
 {
    FILE *fd;
+   char *tmp_s = getenv((const char *)"XCIRCUIT_LIB_DIR");
 
-   sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
+   if (!tmp_s) tmp_s = BUILTINS_DIR;
+   sprintf(_STR2, "%s/%s", tmp_s, STARTUP_FILE);
 
-   if ((fd = fopen(_STR2, "r")) != NULL) readcommand(1, fd);
-   else {
-      sprintf(_STR, "Failed to open startup script \"%s\"\n", STARTUP_FILE);
-      Wprintf(_STR);
+   if ((fd = fopen(_STR2, "r")) == NULL) {
+      sprintf(_STR2, "%s/%s", BUILTINS_DIR, STARTUP_FILE);
+      if ((fd = fopen(_STR2, "r")) == NULL) {
+         sprintf(_STR, "Failed to open startup script \"%s\"\n", STARTUP_FILE);
+         Wprintf(_STR);
+	 return;
+      }
    }
+   readcommand(1, fd);
 }
 
 /*----------------------------------------------------------------------*/
@@ -412,7 +433,7 @@ void execscript()
 {
    FILE *fd;
 
-   tilde_expand(_STR2);
+   xc_tilde_expand(_STR2);
    if ((fd = fopen(_STR2, "r")) != NULL) readcommand(0, fd);
    else {
       sprintf(_STR, "Failed to open script file \"%s\"\n", _STR2);
@@ -434,10 +455,12 @@ void loadrcfile()
 
    /* try first in current directory, then look in user's home directory */
 
-   tilde_expand(_STR2);
+   xc_tilde_expand(_STR2);
    if ((fd = fopen(_STR2, "r")) == NULL) {
-      sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
-      fd = fopen(_STR2, "r");
+      if (userdir != NULL) {
+         sprintf(_STR2, "%s/%s", userdir, USER_RC_FILE);
+         fd = fopen(_STR2, "r");
+      }
    }
    if (fd != NULL) flags = readcommand(0, fd);
 
@@ -464,16 +487,26 @@ void loadrcfile()
       addnewcolorentry(xc_alloccolor("Gray90"));
       addnewcolorentry(xc_alloccolor("Red"));
       addnewcolorentry(xc_alloccolor("Blue"));
-      addnewcolorentry(xc_alloccolor("Green"));
+      addnewcolorentry(xc_alloccolor("Green2"));
       addnewcolorentry(xc_alloccolor("Yellow"));
       addnewcolorentry(xc_alloccolor("Purple"));
-      addnewcolorentry(xc_alloccolor("Cyan"));
-      addnewcolorentry(xc_alloccolor("Magenta"));
-      addnewcolorentry(xc_alloccolor("Orange"));
+      addnewcolorentry(xc_alloccolor("SteelBlue2"));
+      addnewcolorentry(xc_alloccolor("Red3"));
       addnewcolorentry(xc_alloccolor("Tan"));
       addnewcolorentry(xc_alloccolor("Brown"));
    }
 
+#ifdef SCHEMA
+   /* These colors must be enabled whether or not colors are overridden, */
+   /* because they are needed by the schematic capture system.           */
+
+   addnewcolorentry(xc_getlayoutcolor(LOCALPINCOLOR));
+   addnewcolorentry(xc_getlayoutcolor(GLOBALPINCOLOR));
+   addnewcolorentry(xc_getlayoutcolor(INFOLABELCOLOR));
+#endif
+   addnewcolorentry(xc_getlayoutcolor(BBOXCOLOR));
+
+
    /* Add the default key bindings */
 
    if (!(flags & KEYOVERRIDE))
@@ -481,5 +514,5 @@ void loadrcfile()
 }
 
 #endif
-/* #endif for ifndef HAVE_PYTHON */
+/* #endif for if !defined(HAVE_PYTHON) && !defined(TCL_WRAPPER) */
 /*-------------------------------------------------------------------------*/
