@@ -4,17 +4,12 @@
 /* These routines work only if ghostscript is on the system.		*/
 /*----------------------------------------------------------------------*/
 
-/* #undef GS_DEBUG */
-#define GS_DEBUG
+#undef GS_DEBUG
+/* #define GS_DEBUG */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <unistd.h>
 #include <time.h>
 #include <sys/wait.h>   /* for waitpid() */
@@ -134,7 +129,7 @@ void ask_for_next()
    XSync(dpy, False);
    send_client(gvnext);
 #ifdef GS_DEBUG
-   Fprintf(stdout, "Xcircuit: Sent NEXT message to ghostscript\n");
+   fprintf(stdout, "Xcircuit: Sent NEXT message to ghostscript\n");
 #endif
 }
 
@@ -153,7 +148,7 @@ void wait_for_page()
 	 if (event.xclient.message_type == gvpage) break;
          else if (event.xclient.message_type == gvdone) {
 #ifdef GS_DEBUG
-	    Fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
+	    fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
 #endif
 	    /* Catastrophic error in ghostscript */
 	    return;
@@ -174,7 +169,7 @@ void wait_for_page()
       else xcDispatchEvent(&event);	
    }
 #ifdef GS_DEBUG
-   Fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
+   fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
 #endif
    mwin = event.xclient.data.l[0];
 }
@@ -191,13 +186,13 @@ void wait_for_done()
 	 if (event.xclient.message_type == gvdone) break;
 #ifdef GS_DEBUG
          else if (event.xclient.message_type == gvpage)
-	    Fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
+	    fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
 #endif
       }
       else xcDispatchEvent(&event);
    }
 #ifdef GS_DEBUG
-   Fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
+   fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
 #endif
    mwin = 0;
 }
@@ -215,9 +210,30 @@ void start_gs()
    static char env_str1[128], env_str2[64];
 #endif
 
+#ifdef TCL_WRAPPER
+   if (bbuf != (Pixmap)NULL) Tk_FreePixmap(dpy, bbuf);
+#ifdef DOUBLEBUFFER
+   bbuf = Tk_GetPixmap(dpy, dbuf, areastruct.width, areastruct.height,
+		Tk_Depth(areastruct.area));
+#else
+   bbuf = Tk_GetPixmap(dpy, (Drawable)areastruct.areawin,
+		areastruct.width, areastruct.height,
+		Tk_Depth(areastruct.area));
+#endif
+
+#else	/* !TCL_WRAPPER */
    if (bbuf != (Pixmap)NULL) XFreePixmap(dpy, bbuf);
-   bbuf = XCreatePixmap(dpy, areastruct.areawin, areastruct.width, areastruct.height,
-      DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+#ifdef DOUBLEBUFFER
+   bbuf = XCreatePixmap(dpy, dbuf, areastruct.width, areastruct.height,
+		DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+#else
+   bbuf = XCreatePixmap(dpy, areastruct.areawin,
+		areastruct.width, areastruct.height,
+		DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+#endif	/* DOUBLEBUFFER */
+#endif  /* TCL_WRAPPER */
+
+   XSync(dpy, False);
 
    ret = pipe(fgs);
    ret = pipe(std_out);
@@ -234,7 +250,7 @@ void start_gs()
       gsproc = vfork();
       if (gsproc == 0) {		/* child process (gs) */
 #ifdef GS_DEBUG
-         Fprintf(stdout, "Calling %s\n", GS_EXEC);
+         fprintf(stdout, "Calling %s\n", GS_EXEC);
 #endif
 	 close(std_out[0]);
 #ifndef GS_DEBUG
@@ -265,11 +281,10 @@ void start_gs()
 	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
 	 setenv("GHOSTVIEW", _STR, True);
 #else
- 	 sprintf(_STR, "%ld", (long)win);
+ 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
 	 setenv("GHOSTVIEW", _STR, True);
 #endif
 #endif
-	 Fprintf(stderr, "Did putenv \"%s\"\n", env_str2);
 	 Flush(stderr);
          execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", NULL);
 	 gsproc = -1;
@@ -310,7 +325,7 @@ void parse_bg(FILE *fi, FILE *fbg) {
 	       sscanf(bbptr + 12, "%d %d %d %d", &llx, &lly, &urx, &ury);
 	       /* compute user coordinate bounds from PostScript bounds */
 #ifdef GS_DEBUG
-	       Fprintf(stdout, "BBox %d %d %d %d PostScript coordinates\n",
+	       fprintf(stdout, "BBox %d %d %d %d PostScript coordinates\n",
 			llx, lly, urx, ury);
 #endif
 	       llx = (int)((float)llx / psscale);
@@ -318,7 +333,7 @@ void parse_bg(FILE *fi, FILE *fbg) {
 	       urx = (int)((float)urx / psscale);
 	       ury = (int)((float)ury / psscale);
 #ifdef GS_DEBUG
-	       Fprintf(stdout, "BBox %d %d %d %d XCircuit coordinates\n",
+	       fprintf(stdout, "BBox %d %d %d %d XCircuit coordinates\n",
 			llx, lly, urx, ury);
 #endif
 	
@@ -477,7 +492,7 @@ void send_to_gs(char *text)
 {
    write(fgs[1], text, strlen(text));
 #ifdef GS_DEBUG
-   Fprintf(stdout, "writing: %s", text);
+   fprintf(stdout, "writing: %s", text);
 #endif
 }
 
@@ -519,7 +534,7 @@ int renderbackground()
    if (*bgfile == '@') bgfile++;
 
 #ifdef GS_DEBUG
-   Fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
+   fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
 #endif
 
    /* write scale and position to ghostscript 		*/
@@ -583,11 +598,11 @@ int exit_gs()
    send_to_gs("quit\n");
    ask_for_next();
 #ifdef GS_DEBUG
-   Fprintf(stdout, "Waiting for gs to exit\n");
+   fprintf(stdout, "Waiting for gs to exit\n");
 #endif
    waitpid(gsproc, NULL, 0);      
 #ifdef GS_DEBUG
-   Fprintf(stdout, "gs has exited\n");
+   fprintf(stdout, "gs has exited\n");
 #endif
 
    mwin = 0;
