@@ -10,11 +10,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -345,7 +340,7 @@ Boolean pathselect(genericptr *curgen, short class)
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
 
-short *selectobject(short class, objectptr selobj, objinstptr selinst)
+short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
 {
    short	*newselect, *foundtmp, *foundlist;
 
@@ -913,14 +908,14 @@ genericptr recurselect(short class, pushlistptr *seltop)
    selinst = (*seltop)->thisinst;
    selobj = selinst->thisobject;
 
-   if ((selectobj = selectobject(class, selobj, selinst)) != NULL) {
+   if ((selectobj = genobjectselect(class, selobj, selinst)) != NULL) {
       rgen = *(selobj->plist + (*selectobj));
       /* Fprintf(stdout, "%s found in object %s\n", (rgen->type == LABEL) ?
 			"label" : "polygon", selobj->name); */
       drawselects(selobj, selinst);
       clearselects();
    }
-   else if ((selectobj = selectobject(OBJECT, selobj, selinst)) != NULL) {
+   else if ((selectobj = genobjectselect(OBJECT, selobj, selinst)) != NULL) {
       pushlistptr selnew = (pushlistptr)malloc(sizeof(pushlist));
 
       selinst = TOOBJINST(selobj->plist + (*selectobj));	
