@@ -241,7 +241,7 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
 	    if (topobject->params != NULL)
 	       topobject->params[paramno]->parameter.string = dstr->nextpart;
 	    else
-	       fprintf(stderr, "Error in deletestring:  Bad parameter number found\n"); 
+	       Fprintf(stderr, "Error in deletestring:  Bad parameter number found\n"); 
 	 }
 	 else if (thisinst->params == NULL || thisinst->params[paramno] == NULL)
 	    thisinst->thisobject->params[paramno]->parameter.string =
@@ -334,9 +334,13 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 
       if (nextptr == NULL) {
 	 if (localinst->thisobject->params != NULL) {
-	    if (localinst->thisobject->params[paramno]->type != XC_STRING)
-	       goto linkerror;
-            nextptr = localinst->thisobject->params[paramno]->parameter.string;
+	    if (localinst->thisobject->num_params > paramno) {
+	       if (localinst->thisobject->params[paramno]->type != XC_STRING)
+	          goto linkerror;
+               nextptr = localinst->thisobject->params[paramno]->parameter.string;
+	    }
+	    else
+	       return NULL;	/* Error:  There is no parameter here! */
 	 }
 	 else
 	    /* We get here in cases where the object definition is being read,	*/
@@ -360,7 +364,7 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
    return NULL;
 
 linkerror:
-   fprintf(stderr, "linkstring() error: parameter %d is not a substring!\n",
+   Fprintf(stderr, "linkstring() error: parameter %d is not a substring!\n",
 		paramno);
    return strstart->nextpart;
 }
@@ -825,7 +829,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 	       if (topobject->params != NULL)
 	          pparam = topobject->params[paramno];
 	       else
-		  fprintf(stderr, "Error:  Bad parameter number encountered!\n");
+		  Fprintf(stderr, "Error:  Bad parameter number encountered!\n");
 	    }
 	    else if ((thisinst->params) != NULL && (thisinst->params[paramno] != NULL))
 	       pparam = thisinst->params[paramno];
@@ -837,7 +841,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 	    paramno = -1;
 	 }
          else {
-	    fprintf(stderr, "Error:  Bad parameter in stringcopyback()\n");
+	    Fprintf(stderr, "Error:  Bad parameter in stringcopyback()\n");
 	 }
       }
       else
