@@ -118,7 +118,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* selection from top-level object */
 
-#define objectselect(a)	 selectobject(a, topobject, areastruct.topinstance)
+#define objectselect(a)	 genobjectselect(a, topobject, areastruct.topinstance)
 #define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
 #define drawselected(a)	 gendrawselected(a, topobject, areastruct.topinstance)
 
@@ -347,11 +347,21 @@ ULPARAM_MODE, UIPARAM_MODE
 #endif
 
 /*----------------------------------------------------------------------*/
-/* Save types								*/
+/* Save types.	This list incorporates "ALL_PAGES", below.		*/
 /*----------------------------------------------------------------------*/
 
-#define CURRENT_PAGE	0
-#define ALL_PAGES	1
+#define CURRENT_PAGE	0	/* Current page + all associated pages	*/
+#define NO_SUBCIRCUITS	1	/* Current page w/o subcircuit pages	*/
+
+/*----------------------------------------------------------------------*/
+/* Modes used when ennumerating page totals.				*/
+/*----------------------------------------------------------------------*/
+
+#define INDEPENDENT	0
+#define DEPENDENT	1
+#define TOTAL_PAGES	2
+#define PAGE_DEPEND	3
+#define ALL_PAGES	4
 
 /*----------------------------------------------------------------------*/
 /* Color scheme styles (other than NORMAL)				*/
@@ -984,11 +994,11 @@ enum {
    XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
    XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
-   XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc, XCF_Text,
-   XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline, XCF_Edit,
-   XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed, XCF_Dotted,
-   XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start, XCF_Finish,
-   XCF_Cancel, XCF_Nothing, XCF_Exit,
+   XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
+   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline,
+   XCF_Edit, XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
+   XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
+   XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
 
 #ifdef SCHEMA
    XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Pin_Global, XCF_Info_Label,
@@ -1076,6 +1086,7 @@ typedef struct {
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
+   pushlistptr	stack;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
@@ -1110,7 +1121,6 @@ typedef struct {
    Library	fontlib;
    Library	*userlibs;
    objinstptr   *libtop;
-   pushlistptr	stack;
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
@@ -1120,7 +1130,7 @@ typedef struct {
 /* Menu Definitions for hierarchical pulldown menus                     */
 /*----------------------------------------------------------------------*/
 
-/* #ifndef TCL_WRAPPER */	/* to be reinstated: need to compile */
+#ifndef TCL_WRAPPER
 
 typedef struct _menustruct *menuptr;
 
@@ -1144,7 +1154,7 @@ typedef struct _toolbarstruct {
 	char *hint;
 } toolbarstruct;
 
-/* #endif */	/* to be reinstated */
+#endif
 /* Menus and Toolbars are taken care of entirely by scripts in the Tcl/	*/
 /* Tk version of xcircuit.						*/
 /*----------------------------------------------------------------------*/
