@@ -26,6 +26,7 @@ extern int Tk_SimpleObjCmd();
 extern int xctcl_refresh();
 extern int xctcl_library();
 extern int xctcl_font();
+extern int xctcl_bind();
 extern int xctcl_filerecover();
 extern int xctcl_color();
 extern int xctcl_dofill();
@@ -78,6 +79,7 @@ static cmdstruct xc_commands[] =
    {"refresh", (void *)xctcl_refresh},
    {"library", (void *)xctcl_library},
    {"loadfont", (void *)xctcl_font},
+   {"bindkey", (void *)xctcl_bind},
    {"filerecover", (void *)xctcl_filerecover},
    {"color", (void *)xctcl_color},
    {"fill", (void *)xctcl_dofill},
@@ -132,6 +134,7 @@ Xcircuit_Init(Tcl_Interp *interp)
    int cmdidx;
    Tk_Window tktop;
    char *tmp_s;
+   char *cadhome;
 
    /* Interpreter sanity checks */
    if (interp == NULL) return TCL_ERROR;
@@ -185,6 +188,12 @@ Xcircuit_Init(Tcl_Interp *interp)
 
    Tcl_SetVar(interp, "XCIRCUIT_LIB_DIR", tmp_s, TCL_GLOBAL_ONLY);
 
+   /* Set $CAD_HOME as a Tcl variable */
+
+   cadhome = getenv("CAD_HOME");
+   if (cadhome == NULL) cadhome = CAD_DIR;
+   Tcl_SetVar(interp, "CAD_HOME", cadhome, TCL_GLOBAL_ONLY);
+
    /* Export the namespace commands */
 
    Tcl_Eval(interp, "namespace eval xcircuit namespace export *");
